// This file is currently unlocked (change this line if you lock the file)
//
// Source code version is 2.3.0
//                        (increment this number EVERY time you make a change)
//
// 2.3.0        26 Jun 1997     Jill Baker
//      Original version
//
// If this file is unlocked, or locked by someone other than you, then you
// may download this file for incorporation into your build, however any
// changes you make to your copy of this file will be considered volatile,
// and will not be incorporated into the master build which is maintained
// at this web site.
//
// If you wish to make permanent changes to this file, you must wait until
// it becomes unlocked (if it is not unlocked already), and then lock it
// yourself. Whenever the file is locked by you, you may upload new master
// versions to the master site. Make sure you update the history information
// above. When your changes are complete, you should relinquish the lock to
// make it available for other developers.

/*
 * Copyright (C) 1995, 1996 Systemics Ltd (http://www.systemics.com/)
 * All rights reserved.
 *
 * This library and applications are FREE FOR COMMERCIAL AND NON-COMMERCIAL USE
 * as long as the conditions within the COPYRIGHT file are adhered to.
 *
 */

package cryptix.io;

import java.io.BufferedReader;
import java.io.File;
import java.io.InputStreamReader;
import java.io.IOException;
import java.io.OutputStreamWriter;
import java.io.PrintWriter;
import java.util.Date;
import java.util.Vector;

/**
 * @author Jill Baker
 */
public class CommandLineUserInterface extends UserInterface
{
    private static BufferedReader in = new BufferedReader( new InputStreamReader( System.in ));
    private static PrintWriter out = new PrintWriter( new OutputStreamWriter( System.out ), true ); 
    private static long lastUsedTime = (new Date()).getTime();

    private String readLine()
    {
        try
        {
            out.flush();
            String s = in.readLine();
            long now = (new Date()).getTime();
            long interval = now - lastUsedTime;
            addEntropy( interval );
            lastUsedTime = now;
            return s;
        }
        catch (IOException e)
        {
            throw new Error( "IOException: " + e.getMessage() );
        }
    }

    /**
	 * Display a status message
	 * @param message the message to be displayed
	 */
	public void status( String message )
    {
        if (verbose) out.println( message );
    }

	/**
	 * Clear any existing status message
	 */
	public void status(){}

	/**
	 * Notify the user of something
	 * @param message the message to be displayed
	 */
	public void notice( String message )
    {
        out.println( message );
    }

	/**
	 * Warn the user of something
	 * @param message the message to be displayed
	 */
	public void warning( String message )
    {
        out.println( Lang.translate( "Warning" ) + ": " + message );
    }

	/**
	 * Inform the user of an error
	 * @param message the message to be displayed
	 */
	public void error( String message )
    {
        out.println( Lang.translate( "Error" ) + ": " + message );
    }

	/**
	 * Prompt the user for a yes or no answer
	 * @param prompt the question to be displayed
	 * @returns true if yes, false if no
	 */
	public boolean promptYesNo( String prompt, boolean def )
    {
        for (;;)
        {
            out.print( prompt + " (" + y + "/" + n + ")? " );
            String line = readLine();
            if (line.length() == 0) return def;
            if (line.equalsIgnoreCase( y )) return true;
            if (line.equalsIgnoreCase( n )) return false;
        }
    }

	/**
	 * Prompt the user for one of two choices
	 * @param prompt the question to be displayed
     * @param posText text for the positive button
     * @param negText text for the negative button
     * @param def default value
	 * @returns true if the positive button was pressed, false if the negative button was pressed.
	 */
	public boolean promptBoolean( String prompt, String posText, String negText, boolean def )
    {
        for (;;)
        {
            out.print( prompt + " (" + posText + "/" + negText + ")? " );
            String line = readLine();
            if (line.length() == 0) return def;
            if (line.equalsIgnoreCase( posText )) return true;
            if (line.equalsIgnoreCase( negText )) return false;
        }
    }

    /**
     * Prompt the user for a String.
	 * @param prompt the question to be displayed
	 * @returns the String input.
     */
    public String promptString( String prompt )
    {
        out.print( prompt + ": " );
        return readLine();
    }

    /**
     * Prompt the user for a filename.
	 * @param prompt the question to be displayed
     * @param save true if file is to be saved, false if it is to be loaded
	 * @returns the file selected. (This is not guaranteed to exist).
     */
    public File promptFilename( String prompt, boolean save )
    {
        return new File( promptString( prompt ));
    }

    /**
     * Prompt the user for a selection from a list.
	 * @param prompt the question to be displayed
     * @param list Vector of Strings to be selected from
     * @param allowOthers true if user is allowed to type in entries not in the list
	 * @returns the file selected. (This is not guaranteed to exist).
     */
    private static final int SCREEN_SIZE = 20;
    public String promptFromList( String prompt, Vector list, boolean allowOthers )
    {
        int screen = 0;
        int listSize = list.size();
        int numScreens = (listSize - 1) / SCREEN_SIZE + 1;
        for (;;)
        {
            out.println();
            out.println( prompt );
            int start = screen * SCREEN_SIZE;
            for (int i=0; i<SCREEN_SIZE; i++)
            {
                if (start+i < listSize) out.println( (char)('A'+i) + ": " + list.elementAt( start+i ));
            }
            if (numScreens > 1) out.println( "1 - " + (char)('0' + numScreens) + ": " + Lang.translate( "More" ));
            String line = readLine();
            if (line.length() == 0) continue;
            if (allowOthers && line.length() != 1) return line;
            int n = line.charAt(0);
            if (n>='1' && n<='9')
            {
                screen = n - '1';
                continue;
            }
            if (Character.isLowerCase( (char)n )) n = Character.toUpperCase( (char)n );
            n -= 'A';
            if (n<0 || n>=SCREEN_SIZE) continue;
            return (String)list.elementAt( start+n );
        }
    }

	/**
	 * Prompt the user for a passphrase.
	 * @param s the prompt to be displayed
     * @charset charset for character conversion
	 */
	public void getPassphrase( String prompt, String charset )
    {
        String phrase = promptString( prompt );
        for (int i=0; i<phrase.length(); i++)
        {
            addPassphraseChar( phrase.charAt(i), charset );
        }
    }

    /**
     * Prompt the user for some entropy.
     */
    public void getEntropy( int numBytes )
    {
        String s = promptString( "Please enter some random text on your keyboard" );
        for (int i=0; i<s.length(); i++) addEntropy( s.charAt(i) );
    }
}
