// This file is currently unlocked (change this line if you lock the file)
//
// Source code version is 2.5.8
//                        (increment this number EVERY time you make a change)
//
// History: (add to this EVERY time you make a change) 
// (insert details of your change immediately below this line)
//
// 2.5.8        12 Jun 1997     Jill Baker
//      Support added for unencrypted secret keys. 
//      Added methods getSecretKey() and getAlgorithm().
//
// 2.5.7        11 Jun 1997     Jill Baker
//      Uses new Passphrase.hash() method. Added new method getSecretKey( byte[] ).
//
// 2.5.6        07 Jun 1997     Ian Brown
//      added bitLength() functions
//
// 2.5.5        27 May 1997     Jill Baker
//      Successful decryption using a Passphrase now touch()es the Passphrase.
//
// 2.5.4        23 May 1997     Jill Baker
//      Passes on exceptions from Passphrase
//
// 2.5.3	    22 May 1997	    Ian Brown
//	    Uses new Passphrase.md5hash() method
//
// 2.5.2        10 Apr 1997     Jill Baker
//      added method getType()
//      imports changed to new namespace
//
// 2.5.1        01 Apr 1997     Jill Baker
//      Modified to work with new version of Passphrase
//
// 2.5.0        24 Mar 1997     Original Author not stated
//      Original version
//
// If this file is unlocked, or locked by someone other than you, then you
// may download this file for incorporation into your build, however any
// changes you make to your copy of this file will be considered volatile,
// and will not be incorporated into the master build which is maintained
// at this web site.
//
// If you wish to make permanent changes to this file, you must wait until
// it becomes unlocked (if it is not unlocked already), and then lock it
// yourself. Whenever the file is locked by you, you may upload new master
// versions to the master site. Make sure you update the history information
// above. When your changes are complete, you should relinquish the lock to
// make it available for other developers.

/*
 * Copyright (C) 1995, 1996 Systemics Ltd (http://www.systemics.com/)
 * All rights reserved.
 *
 */

//
//	This module still has IDEA hard coded all over the place
//	This must be fixed
//

package cryptix.pgp;

import java.io.DataInput;
import java.io.DataOutput;
import java.io.IOException;
import cryptix.math.BigInteger;
import cryptix.math.MPI;
import cryptix.math.RandomStream;
import cryptix.security.StreamCipher;
import cryptix.security.MD5;
import cryptix.security.IDEA;
import cryptix.security.rsa.SecretKey;
import cryptix.util.SensitiveByteArray;
import cryptix.util.Streamable;

/**
 *
 * <hr>
 * <P>
 * <bold>Copyright</bold> &#169 1995, 1996, 1997
 * <a href="http://www.systemics.com/">Systemics Ltd</a>
 * on behalf of the
 * <a href="http://www.systemics.com/docs/cryptix/">
 * Cryptix Development Team</a>.
 * All rights reserved.
 */
final class EncryptedBigInt extends Streamable
{
	private byte buf_[];
	private int bitLen_;
	private int checksum_;

	EncryptedBigInt( DataInput in )
		throws IOException
	{
		super( in );
	}
	
	EncryptedBigInt( BigInteger num, StreamCipher cipher )
	{
		buf_ = num.toByteArray();
		bitLen_ = num.bitLength();
		int len = buf_.length;
		int sum = 0;
		for (int i = len - 1; i >= 0; i-- )
			sum += ( buf_[i] & 0xFF );
		checksum_ = sum & 0xFFFF;
		cipher.encrypt( buf_ );
	}
	
	BigInteger
	decrypt( StreamCipher cipher )
	{
		int len = buf_.length;
		byte tmp[] = new byte[len];
		cipher.decrypt( buf_, tmp );
		
		int sum = 0;
		for (int i = len - 1; i >= 0; i-- )
			sum += ( tmp[i] & 0xFF );

		checksum_ = sum & 0xFFFF;
	
		return new BigInteger( tmp );
	}
	
	public void
	read( DataInput in )
		throws IOException
	{
		bitLen_ = in.readShort();
		buf_ = new byte[(bitLen_ + 7) / 8];
		in.readFully( buf_ );
	}

	public void
	write( DataOutput out )
		throws IOException
	{
		out.writeShort( bitLen_ );
		out.write( buf_ );
	}
	
	public int
	checksum()
	{
		return ( ( bitLen_ & 0xFF ) + ( ( bitLen_ >> 8 ) & 0xFF ) + checksum_ ) & 0xFFFF;
	}
}

final class EncryptedSecretKey extends Streamable
{
    private int algorithm;
	private BigInteger e_, n_;
    private BigInteger d, p, q, u;              // if unencrypted
	private EncryptedBigInt d_, p_, q_, u_;     // if encrypted
	private byte iv_[] = null;
	private int checksum_;

	public EncryptedSecretKey( SecretKey sk, Passphrase passphrase, RandomStream rs )
		throws IOException, DecryptException
	{
        SensitiveByteArray sba = passphrase.hash();
        IDEA block_cipher = new IDEA( sba.getData() );
        sba.burn();
		CFB cipher = new CFB(block_cipher);

		n_ = sk.n();
		e_ = sk.e();

		rs.read( iv_ = new byte[block_cipher.blockLength()] );
		cipher.encrypt(iv_);

		d_ = new EncryptedBigInt( sk.d(), cipher );
		p_ = new EncryptedBigInt( sk.p(), cipher );
		q_ = new EncryptedBigInt( sk.q(), cipher );
		u_ = new EncryptedBigInt( sk.u(), cipher );

		checksum_ = ( d_.checksum() + p_.checksum() + q_.checksum() + u_.checksum() ) & 0xFFFF;
	}

	public EncryptedSecretKey( byte buf[] )
		throws IOException
	{
		load( buf );
	}

	public EncryptedSecretKey( DataInput in )
		throws IOException
	{
		read( in );
	}

    public SecretKey secretKey() throws DecryptException
	{
        if (algorithm != Algorithm.NONE)
            throw new DecryptException( "Passphrase needed to unlock secret key" );
        return new SecretKey( n_, e_, d, p, q, u );
	}
		
    public SecretKey secretKey( StreamCipher cipher ) throws DecryptException
	{
        if (algorithm != Algorithm.IDEA)
			throw new DecryptException( "Unknown cipher type protecting secret key" );

        cipher.decrypt( iv_, new byte[iv_.length] );

		BigInteger d = d_.decrypt( cipher );
		BigInteger p = p_.decrypt( cipher );
		BigInteger q = q_.decrypt( cipher );
		BigInteger u = u_.decrypt( cipher );

		if ( ( ( ( d_.checksum() + p_.checksum() + q_.checksum() + u_.checksum() ) ) & 0xFFFF ) != checksum_ )
			throw new DecryptException( "Check sum failure: passphrase wrong." );
		return new SecretKey( n_, e_, d, p, q, u );
	}
		
	public void read(DataInput in)
		throws IOException
	{
		n_ = MPI.read( in );
		e_ = MPI.read( in );
        algorithm = in.readByte();

        switch (algorithm)
        {
            case Algorithm.NONE:
            {
                d = MPI.read( in );
                p = MPI.read( in );
                q = MPI.read( in );
                u = MPI.read( in );

                checksum_ = in.readShort() & 0xFFFF;
                break;
            }
            case Algorithm.IDEA:
            {
			    in.readFully( iv_ = new byte [8] ); // IV for cipher.

			    d_ = new EncryptedBigInt( in );
			    p_ = new EncryptedBigInt( in );
			    q_ = new EncryptedBigInt( in );
			    u_ = new EncryptedBigInt( in );

			    checksum_ = in.readShort() & 0xFFFF;
                break;
            }
            default:
            {
				throw new FormatException( "Unknown cipher type protecting secret key" );
            }
        }

        /*
        int encrypted = in.readByte();
		if ( encrypted != 0 )
		{
			if ( encrypted != 1 ) // IDEA
				throw new FormatException( "Unknown cipher type" );

			in.readFully( iv_ = new byte [8] ); // IV for cipher.

			d_ = new EncryptedBigInt( in );
			p_ = new EncryptedBigInt( in );
			q_ = new EncryptedBigInt( in );
			u_ = new EncryptedBigInt( in );

			checksum_ = in.readShort() & 0xFFFF;
		}
		else
			throw new FormatException( "Found unencrypted SecretKey - unencrypted secret keys are not supported." );
        */
	}
	
	public void write( DataOutput out )
		throws IOException
	{
		MPI.write( out, n_ );
		MPI.write( out, e_ );
		out.write( 1 );
		out.write( iv_ );

		d_.write( out );
		p_.write( out );
		q_.write( out );
		u_.write( out );

		out.writeShort( checksum_ );
	}

	/**
	 * gets the key id. Needed to index secret key certificates by ID.
	 * @returns The lower 8 bytes of n, the public modulus.
	 */
	public final KeyID
	id()
	{
		byte nBuffer[] = n_.toByteArray();
		byte ident[] = new byte[8];
		System.arraycopy( nBuffer, nBuffer.length - 8, ident, 0, 8 );
		return new KeyID( ident );
	}

	
	public int
	bitLength()
	{
		return n_.bitLength();
	}
	
	public String
	toString()
	{
		return "secret key public parts n:" + n_.toString() + " e:"+ e_.toString() + " all the rest is encrypted";
	}

    public int getAlgorithm()
    {
        return algorithm;
    }
}

public final class SecretKeyCertificate extends Certificate 
{
	private EncryptedSecretKey key;
	
	public SecretKeyCertificate( SecretKey key0, Passphrase passphrase, RandomStream rand )
		throws IOException, DecryptException
	{
		key = new EncryptedSecretKey( key0, passphrase, rand );
	}

	public SecretKeyCertificate( DataInput in, int length )
		throws IOException
	{
		super( in, length );
	}

	public void
	read( DataInput in, int length )
		throws IOException
	{
		super.read( in, length );
		
		if ( in.readByte() != 0x01 )
			throw new FormatException( "Bad algorithm byte (not RSA)." );

		key = new EncryptedSecretKey( in );
	}
	
	public int
	write( DataOutput out )
		throws IOException
	{
		int len = super.write( out ) + 1;
		out.write( 0x01 );
		byte tmp[];
		out.write( tmp = key.save() );
		len += tmp.length;
		return len;
	}

    public SecretKey getSecretKey() throws DecryptException
    {
        return key.secretKey();
    }

	public SecretKey
	getSecretKey( Passphrase passphrase )
		throws DecryptException
	{
        SensitiveByteArray sba = passphrase.hash();
        CFB cipher = new CFB( new IDEA( sba.getData() ));
        sba.burn();
        SecretKey secretKey = key.secretKey( cipher );
        passphrase.touch();
        return secretKey;
	}

    public SecretKey getSecretKey( byte[] hash ) throws DecryptException
	{
        return key.secretKey( new CFB( new IDEA( hash )));
	}

	/**
	 * gets the key id.
	 * @returns The key in the certificate's ID
	 */
	public final KeyID
	id()
	{
		return key.id();
	}

	public String
	toString()
	{
		return "Secret key Cert " + super.toString();
	}
	
	public int
	bitLength()
	{
		return key.bitLength();
	}

    public int getType()
    {
        return Packet.SECRET_KEY;
    }

    public int getAlgorithm()
    {
        return key.getAlgorithm();
    }
}
