// This file is currently unlocked (change this line if you lock the file)
//
// Source code version is 2.5.6
//                        (increment this number EVERY time you make a change)
//
// History: (add to this EVERY time you make a change) 
// (insert details of your change immediately below this line)
//
// 2.5.6        30 Jun 1997     Ian Brown
//      put DataInputStream.readLine()back so will compile under Java 1.0x
//	removed comments - everything seems fine
//
// 2.5.5        13 Jun 1997     Jill Baker
//      Call to deprecated method DataInputStream.readLine() updated to J1.1.
//
// 2.5.4        11 Jun 1997     Jill Baker
//      Temporary bug fix. Some lines commented out (104-114) as they were
//      causing a compile failure. Ian can sort it out properly when he gets back.
//
// 2.5.3	7 June 1997	Ian Brown
//	changed return types of fetch() functions from to PublicKey to KeyRingEntry
//	to allow for caching and adding retrieved keys to keyrings.
//
// 2.5.2        29 Apr 1997     Ian Brown
//      changed constructor to take the path to the CGI keyserver
//	changed return types of fetch() functions from KeyRingEntry to PublicKey
//
// 2.5.1        15 Apr 1997     Jill Baker
//      imports changed to new namespace
//
// 2.5.0        29 Dec 1996     Ian Brown
//      Original version
//
// If this file is unlocked, or locked by someone other than you, then you
// may download this file for incorporation into your build, however any
// changes you make to your copy of this file will be considered volatile,
// and will not be incorporated into the master build which is maintained
// at this web site.
//
// If you wish to make permanent changes to this file, you must wait until
// it becomes unlocked (if it is not unlocked already), and then lock it
// yourself. Whenever the file is locked by you, you may upload new master
// versions to the master site. Make sure you update the history information
// above. When your changes are complete, you should relinquish the lock to
// make it available for other developers.

package cryptix.pgp;

import java.io.*;
import java.net.*;

import cryptix.security.rsa.PublicKey;
import cryptix.security.rsa.SecretKey;
import cryptix.util.EmailAddress;

/**
 * @version 2.0 29/12/1996
 * @author Ian Brown
 *
 * <hr>
 * <P>
 * <bold>Copyright</bold> &#169 1995, 1996, 1997
 * <a href="http://www.systemics.com/">Systemics Ltd</a>
 * on behalf of the
 * <a href="http://www.systemics.com/docs/cryptix/">
 * Cryptix Development Team</a>.
 * All rights reserved.
 */
public final class WebClient extends KeyClient
{
	private String pathToPage;
	
	public WebClient( String name, int port_, String path  )
	throws IOException, UnknownHostException
	{
		super( name, port_ );
		pathToPage = path;
	}


	/**
	 * Called by KeyClient.getKey() to fetch a key using a specific protocol
	 * Should NOT be called directly - this would bypass checks and caching
	 */
	protected KeyRingEntry fetch( String userName )
	throws IOException 
	{
		boolean fullKeyReceived = false;
		DataInputStream serverIn = new DataInputStream( new URL( "http", serverName, port, pathToPage + "?op=get&exact=on&search=" + userName ).openStream() );
		String line = null;
		String keyBlock = "";
		
		// Skip HTML formatting and information about key
		do
		{
			line = serverIn.readLine();
			if ( line == null ) return null; // No key returned
		} while ( !line.equals( "-----BEGIN PGP PUBLIC KEY BLOCK-----" ) );
		keyBlock += line + Armoury.LINEFEED;	
		
		// Get rest of key
		while ( ( line = serverIn.readLine() ) != null )
		{
			if ( line.equals( "-----END PGP PUBLIC KEY BLOCK-----" ) )
			{
				keyBlock += line;
				fullKeyReceived = true;
				break; // End of key
			}
			else keyBlock += line + Armoury.LINEFEED;
		}
                try
                {
			if ( ! fullKeyReceived ) return null; // Problem retrieving key
			else return new KeyRingEntry( new PacketInputStream( new ByteArrayInputStream( Armoury.strip( keyBlock ) ) ), KeyRingTrust.UNDEF );
                }

               catch( InvalidChecksumException ice )
               {
                       // We can't throw one as KeyStore (the ultimate parent class ) doesn't allow it
                       // Armoury.strip may throw it when removing the ASCII armour on the key block
                       throw new IOException( "Key's checksum is incorrect." );
               }
	}

	/**
	 * WebClient doesn't need an address as it queries a
	 * master keyserver.
	 */
	protected KeyRingEntry fetch( EmailAddress address, KeyID keyID )
	throws IOException
	{
		return fetch( keyID.toString() );
	}
	
	protected KeyRingEntry fetch( EmailAddress address )
	throws IOException
	{
		return fetch( address.toString() );
	}
	
	protected KeyRingEntry fetch( KeyID keyID )
	throws IOException
	{
		return fetch( null, keyID );
	}

}
