// This file is currently unlocked (change this line if you lock the file)
//
// Source code version is 2.5.4
//                        (increment this number EVERY time you make a change)
//
// History: (add to this EVERY time you make a change) 
// (insert details of your change immediately below this line)
//
// 2.5.4        02 Jul 1997     Jill Baker
//      Added single byte conversion
//
// 2.5.3        28 Mar 1997     Jill Baker
//      Complete rewrite
//
// 2.5.2        26 Mar 1997     Jill Baker
//      Complete rewrite.
//
// 2.5.1        24 Mar 1997     Jill Baker
//      Original version
//
// If this file is unlocked, or locked by someone other than you, then you
// may download this file for incorporation into your build, however any
// changes you make to your copy of this file will be considered volatile,
// and will not be incorporated into the master build which is maintained
// at this web site.
//
// If you wish to make permanent changes to this file, you must wait until
// it becomes unlocked (if it is not unlocked already), and then lock it
// yourself. Whenever the file is locked by you, you may upload new master
// versions to the master site. Make sure you update the history information
// above. When your changes are complete, you should relinquish the lock to
// make it available for other developers.

package cryptix.pgp.charset;

import cryptix.pgp.*;

/**
 * This abstract class contains static methods to convert between PGP's
 * internal character set, and any charset whose name is recognised
 * by old-PGP.
 *
 * @author Jill Baker
 */
public abstract class CharSet
{
    /**
     * Convert a byte[] buffer from PGP's internal form to the specified
     * charset.
     */
    public static byte[] encode( byte[] in, String name ) throws FormatException
    {
        String lname = name.toLowerCase();

             if (lname.equals( "ascii" ))       return Ascii.encode( in );
        else if (lname.equals( "alt_codes" ))   return Cp866.encode( in );
        else if (lname.equals( "cp850" ))       return Cp850.encode( in );
        else if (lname.equals( "cp852" ))       return Cp852.encode( in );
        else if (lname.equals( "cp860" ))       return Cp860.encode( in );
        else if (lname.equals( "cp866" ))       return Cp866.encode( in );
        else if (lname.equals( "keybcs" ))      return Keybcs.encode( in );
        else if (lname.equals( "ko18" ))        return in;
        else if (lname.equals( "latin1" ))      return in;
        else if (lname.equals( "mac" ))         return Mac.encode( in );
        else if (lname.equals( "next" ))        return Next.encode( in );
        else if (lname.equals( "noconv" ))      return in;

        else throw new FormatException( "Charset '" + name + "' not recognised" );
    }

    /**
     * Convert a byte from PGP's internal form to the specified
     * charset.
     */
    public static byte encode( byte in, String name ) throws FormatException
    {
        String lname = name.toLowerCase();

             if (lname.equals( "ascii" ))       return Ascii.encode( in );
        else if (lname.equals( "alt_codes" ))   return Cp866.encode( in );
        else if (lname.equals( "cp850" ))       return Cp850.encode( in );
        else if (lname.equals( "cp852" ))       return Cp852.encode( in );
        else if (lname.equals( "cp860" ))       return Cp860.encode( in );
        else if (lname.equals( "cp866" ))       return Cp866.encode( in );
        else if (lname.equals( "keybcs" ))      return Keybcs.encode( in );
        else if (lname.equals( "ko18" ))        return in;
        else if (lname.equals( "latin1" ))      return in;
        else if (lname.equals( "mac" ))         return Mac.encode( in );
        else if (lname.equals( "next" ))        return Next.encode( in );
        else if (lname.equals( "noconv" ))      return in;

        else throw new FormatException( "Charset '" + name + "' not recognised" );
    }

    /**
     * Convert a byte[] buffer from the specified charset
     * to PGP's internal form.
     */
    public static byte[] decode( byte[] in, String name ) throws FormatException
    {
        String lname = name.toLowerCase();

             if (lname.equals( "ascii" ))       return Ascii.decode( in );
        else if (lname.equals( "alt_codes" ))   return Cp866.decode( in );
        else if (lname.equals( "cp850" ))       return Cp850.decode( in );
        else if (lname.equals( "cp852" ))       return Cp852.decode( in );
        else if (lname.equals( "cp860" ))       return Cp860.decode( in );
        else if (lname.equals( "cp866" ))       return Cp866.decode( in );
        else if (lname.equals( "keybcs" ))      return Keybcs.decode( in );
        else if (lname.equals( "ko18" ))        return in;
        else if (lname.equals( "latin1" ))      return in;
        else if (lname.equals( "mac" ))         return Mac.decode( in );
        else if (lname.equals( "next" ))        return Next.decode( in );
        else if (lname.equals( "noconv" ))      return in;

        else throw new FormatException( "Charset '" + name + "' not recognised" );
    }

    /**
     * Convert a byte from the specified charset
     * to PGP's internal form.
     */
    public static byte decode( byte in, String name ) throws FormatException
    {
        String lname = name.toLowerCase();

             if (lname.equals( "ascii" ))       return Ascii.decode( in );
        else if (lname.equals( "alt_codes" ))   return Cp866.decode( in );
        else if (lname.equals( "cp850" ))       return Cp850.decode( in );
        else if (lname.equals( "cp852" ))       return Cp852.decode( in );
        else if (lname.equals( "cp860" ))       return Cp860.decode( in );
        else if (lname.equals( "cp866" ))       return Cp866.decode( in );
        else if (lname.equals( "keybcs" ))      return Keybcs.decode( in );
        else if (lname.equals( "ko18" ))        return in;
        else if (lname.equals( "latin1" ))      return in;
        else if (lname.equals( "mac" ))         return Mac.decode( in );
        else if (lname.equals( "next" ))        return Next.decode( in );
        else if (lname.equals( "noconv" ))      return in;

        else throw new FormatException( "Charset '" + name + "' not recognised" );
    }
}



