// This file is currently unlocked (change this line if you lock the file)
//
// Source code version is 2.3.0
//                        (increment this number EVERY time you make a change)
//
// History: (add to this EVERY time you make a change) 
// (insert details of your change immediately below this line)
//
// 2.3.0        16 May 1997     Jill Baker
//      Original Version
//
// If this file is unlocked, or locked by someone other than you, then you
// may download this file for incorporation into your build, however any
// changes you make to your copy of this file will be considered volatile,
// and will not be incorporated into the master build which is maintained
// at this web site.
//
// If you wish to make permanent changes to this file, you must wait until
// it becomes unlocked (if it is not unlocked already), and then lock it
// yourself. Whenever the file is locked by you, you may upload new master
// versions to the master site. Make sure you update the history information
// above. When your changes are complete, you should relinquish the lock to
// make it available for other developers.

/*
 * Copyright (C) 1995, 1996 Systemics Ltd (http://www.systemics.com/)
 * All rights reserved.
 *
 * This library and applications are FREE FOR COMMERCIAL AND NON-COMMERCIAL USE
 * as long as the conditions within the COPYRIGHT file are adhered to.
 *
 */

package cryptix.pgp.engine;

import cryptix.io.Lang;
import cryptix.pgp.*;
import cryptix.pgp.charset.CharSet;
import cryptix.pgp.engine.toolkit.*;
import cryptix.mime.CRLF;
import cryptix.security.MD5;
import cryptix.security.rsa.SecretKey;
import java.io.DataOutputStream;
import java.io.IOException;
import java.io.PrintWriter;
import java.util.Properties;

/**
 * @author Jill Baker
 */
public final class ArmourEngine extends Engine
{
    private ArmouredData armoured;

    /**
     * Constructor which removes armour.
     */
    public ArmourEngine( byte[] armouredData, Toolkit toolkit ) throws FormatException, DecryptException
    {
        toolkit.ui.status( Lang.translate( "Removing Armour" ));
        armoured = new ArmouredData( armouredData );
    }

    /**
     * Constructor which armours an existing engine.
     * @param toolkit toolkit defining user preferences
     * @engine Engine object to armour
     * @return Engine, now armoured
     */
    public ArmourEngine( Toolkit toolkit, Engine engine ) throws IOException
    {
        toolkit.ui.status( Lang.translate( "Armouring" ));
        Properties properties = new Properties();
        properties.put( "Charset", toolkit.charset );
        armoured = new ArmouredData( properties, engine.toPacketByteArray() );
    }

    /**
     * Constructor which clearsigns plaintext
     */
    public ArmourEngine( String plaintext, Toolkit toolkit, SecretKey key )
        throws FormatException, IOException
    {
        toolkit.ui.status( Lang.translate( "Clearsigning" ));
        Properties properties = new Properties();
        properties.put( "Charset", toolkit.charset );
        armoured = new ArmouredData( properties, plaintext, key );
    }

    /**
     * Single stage of decryption process.
     */
    public DecryptStep decryptStep( Toolkit toolkit ) throws DecryptException, IOException
    {
        byte[] clearText = armoured.getClearText();
        if (clearText != null)
        {
            Properties p = armoured.getProperties();
            String originalCharSet = p.getProperty( "charset", "noconv" );
            Utility.dPrintln( "Converting from original charset (" + originalCharSet + ")" );
            clearText = CharSet.decode( clearText, originalCharSet );
            Utility.dPrintln( "Removing trailing spaces" );
            clearText = CRLF.removeTrailingSpaces( clearText );
        }
        toolkit.plaintextGetter = new SimplePlaintextGetter( clearText );
        return new DecryptStep( armoured.getPayload() );
    }

    /**
     * This method is required because ArmourEngine extends Engine.
     * However, it is meaningless in this case, so just return null.
     */
    public PacketByteArray toPacketByteArray() throws IOException
    {
        return null;
    }

    public String toString()
    {
        return armoured.toString();
    }

    /**
     * A real (though very simple) command line PGP armourer!
     */
    public static void main( String[] arg ) throws IOException, DecryptException
    {
        // Uncomment the following line to enable debugging.
        Utility.debug = new PrintWriter( System.out, true );

        // Gather all the details together
        Toolkit toolkit = new Toolkit( arg[0] );
        Engine engine = toolkit.getEngine();

        // Armour it...
        engine = new ArmourEngine( toolkit, engine );

        // Output the file.
        toolkit.setOutputStream( ".asc", (arg.length==1) ? null : arg[1] );
        DataOutputStream dos = new DataOutputStream( toolkit.outputStream );
        dos.writeBytes( engine.toString() );
        dos.close();

        // All done. Tell the user.
        toolkit.ui.notice( Lang.translate( "Armoured file" ) + ": '" + toolkit.destFilename + "'" );
    }

}
