// This file is currently unlocked (change this line if you lock the file)
//
// Source code version is 2.3.0
//                        (increment this number EVERY time you make a change)
//
// History: (add to this EVERY time you make a change) 
// (insert details of your change immediately below this line)
//
// 2.3.0        16 May 1997     Jill Baker
//      Original Version
//
// If this file is unlocked, or locked by someone other than you, then you
// may download this file for incorporation into your build, however any
// changes you make to your copy of this file will be considered volatile,
// and will not be incorporated into the master build which is maintained
// at this web site.
//
// If you wish to make permanent changes to this file, you must wait until
// it becomes unlocked (if it is not unlocked already), and then lock it
// yourself. Whenever the file is locked by you, you may upload new master
// versions to the master site. Make sure you update the history information
// above. When your changes are complete, you should relinquish the lock to
// make it available for other developers.

/*
 * Copyright (C) 1995, 1996 Systemics Ltd (http://www.systemics.com/)
 * All rights reserved.
 *
 * This library and applications are FREE FOR COMMERCIAL AND NON-COMMERCIAL USE
 * as long as the conditions within the COPYRIGHT file are adhered to.
 *
 */

package cryptix.pgp.engine;

import cryptix.io.Lang;
import cryptix.pgp.*;
import cryptix.pgp.engine.toolkit.*;
import java.io.ByteArrayOutputStream;
import java.io.DataOutputStream;
import java.io.IOException;
import java.io.PrintWriter;

/**
 * @author Jill Baker
 */
public final class CompressionEngine extends Engine implements Algorithm
{
    private CompressedData packet;

    /**
     * Constuctor which merely puts all the elements together,
     * assuming they are correct.
     */
    public CompressionEngine( CompressedData packet )
    {
        this.packet = packet;
    }

    /**
     * Constructor which compresses an existing engine.
     * @param toolkit toolkit defining user preferences
     * @engine Engine object to compress
     * @return Engine, now compressed
     */
    public CompressionEngine( Toolkit toolkit, Engine engine ) throws IOException, FormatException
    {
        this( toolkit, engine, toolkit.config.getProperty( "compressionalgorithm", "zip" ) );
    }

    /**
     * Constructor which compresses an existing engine.
     * @param toolkit toolkit defining user preferences
     * @engine Engine object to compress
     * @return Engine, now compressed
     */
    public CompressionEngine( Toolkit toolkit, Engine engine, String algorithmName )
        throws IOException, FormatException
    {
        int algorithm = 0;
        algorithmName = algorithmName.toLowerCase();
        if (algorithmName.equals( "zip" ))
        {
            algorithm = ZIP;
        }
        else
        {
            throw new FormatException( "Unknown compression algorithm (" + algorithm + ")" );
        }
        packet = new CompressedData( engine.toPacketByteArray(), algorithm );
    }

    public DecryptStep decryptStep( Toolkit toolkit ) throws DecryptException, IOException
    {
        toolkit.ui.status( Lang.translate( "Decompressing" ));
        return new DecryptStep( packet.getPacketData() );
    }

    public PacketByteArray toPacketByteArray() throws IOException
    {
        ByteArrayOutputStream out = new ByteArrayOutputStream();
        DataOutputStream dout = new DataOutputStream( out );
        PacketOutputStream pout = new PacketOutputStream( dout );
        pout.writePacket( packet );
        pout.close();
        dout.close();
        return new PacketByteArray( out.toByteArray() );
    }

    /**
     * A real (though very simple) command line PGP compressor!
     */
    public static void main( String[] arg ) throws IOException, DecryptException, FormatException
    {
        // Uncomment the following line to enable debugging.
        Utility.debug = new PrintWriter( System.out, true );

        // Gather all the details together
        Toolkit toolkit = new Toolkit( arg[0] );
        Engine engine = toolkit.getEngine();

        // Compress it...
        engine = new CompressionEngine( toolkit, engine );

        // Output the file
        toolkit.setOutputStream( ".pgp", (arg.length==1) ? null : arg[1] );
        toolkit.outputStream.write( engine.toPacketByteArray().toByteArray() );

        // All done. Tell the user.
        toolkit.ui.notice( Lang.translate( "Compressed file" ) + ": '" + toolkit.destFilename + "'" );
    }
}
