// This file is currently unlocked (change this line if you lock the file)
//
// Source code version is 2.3.0
//                        (increment this number EVERY time you make a change)
//
// History: (add to this EVERY time you make a change) 
// (insert details of your change immediately below this line)
//
// 2.3.0        16 May 1997     Jill Baker
//      Original Version
//
// If this file is unlocked, or locked by someone other than you, then you
// may download this file for incorporation into your build, however any
// changes you make to your copy of this file will be considered volatile,
// and will not be incorporated into the master build which is maintained
// at this web site.
//
// If you wish to make permanent changes to this file, you must wait until
// it becomes unlocked (if it is not unlocked already), and then lock it
// yourself. Whenever the file is locked by you, you may upload new master
// versions to the master site. Make sure you update the history information
// above. When your changes are complete, you should relinquish the lock to
// make it available for other developers.

/*
 * Copyright (C) 1995, 1996 Systemics Ltd (http://www.systemics.com/)
 * All rights reserved.
 *
 * This library and applications are FREE FOR COMMERCIAL AND NON-COMMERCIAL USE
 * as long as the conditions within the COPYRIGHT file are adhered to.
 *
 */

package cryptix.pgp.engine;

import cryptix.io.Lang;
import cryptix.pgp.*;
import cryptix.pgp.engine.toolkit.*;
import cryptix.security.IDEA;
import cryptix.util.SensitiveByteArray;
import java.io.ByteArrayOutputStream;
import java.io.DataOutputStream;
import java.io.IOException;
import java.io.PrintWriter;
import java.util.Enumeration;
import java.util.Vector;


/**
 * @author Jill Baker
 */
public final class ConventionalEncryptionEngine extends Engine
{
    private ConvEncryptedData packet;

    /**
     * Constuctor which merely puts all the elements together,
     * assuming they are correct.
     */
    public ConventionalEncryptionEngine( ConvEncryptedData packet )
    {
        this.packet = packet;
    }

    /**
     * Constructor which conventionally encrypts an existing engine.
     * @param toolkit toolkit defining user preferences
     * @engine Engine object to encrypt
     * @return Engine, now encrypted
     */
    public ConventionalEncryptionEngine( Toolkit toolkit, Engine engine ) throws IOException, FormatException
    {
        this( toolkit, engine, null );
    }

    /**
     * Constructor which conventionally encrypts an existing engine.
     * @param toolkit toolkit defining user preferences
     * @engine Engine object to encrypt
     * @passphrase hash of passphrase with which to encrypt
     * @return Engine, now encrypted
     */
    public ConventionalEncryptionEngine( Toolkit toolkit, Engine engine, SensitiveByteArray passphrase )
        throws IOException, FormatException
    {
        if (passphrase == null)
        {
            SensitiveByteArray sba;
            for (;;)
            {
                passphrase = toolkit.ui.promptPassphrase( toolkit.charset );
                sba = toolkit.ui.promptPassphrase( Lang.translate( "Enter same passphrase again" ), toolkit.charset );
                if ((passphrase != null) && (passphrase.equals( sba )))
                {
                    if (sba != null) sba.burn();
                    break;
                }
                else
                {
                    if (passphrase != null) passphrase.burn();
                    if (sba != null) sba.burn();
                    toolkit.ui.error( Lang.translate( "Passphrases were different" )
                        + ". " + Lang.translate( "Try again" ));
                }
            }
        }
        packet = new ConvEncryptedData( engine.toPacketByteArray().toByteArray(), passphrase.getData(), toolkit.randomStream );
        passphrase.burn();
    }

    public DecryptStep decryptStep( Toolkit toolkit ) throws DecryptException, IOException
    {
        toolkit.ui.status( Lang.translate( "Decrypting conventionally encrypted data" ));
        Vector expiredPassphrases = new Vector();
        Passphrase passphrase;
        Enumeration e1;
        PacketByteArray pba = null;

        generatePBA:
        {
            ConvEncHashTester test = new ConvEncHashTester( packet );
            if (toolkit.findPassphrase( test ))
            {
                pba = test.pba;
                break generatePBA;
            }

            // Input Passphrase manually
            toolkit.ui.notice( Lang.translate( "You will need a passphrase with which to decrypt this file" ));
            for (int i=0; i<3; i++)
            {
                SensitiveByteArray passphraseHash = toolkit.ui.promptPassphrase( toolkit.charset );
                if (passphraseHash == null) return null;
                if (test.testHash( passphraseHash.getData() ))
                {
                    pba = test.pba;
                    toolkit.passphraseVector.addElement( new Passphrase( passphraseHash, toolkit.passphraseTimeout ) );
                    break generatePBA;
                }
                passphraseHash.burn();
            }
            toolkit.ui.error( Lang.translate( "Bad passphrase" ));
            return null;
        }
        return new DecryptStep( pba );
    }

    public PacketByteArray toPacketByteArray() throws IOException
    {
        ByteArrayOutputStream out = new ByteArrayOutputStream();
        DataOutputStream dout = new DataOutputStream( out );
        PacketOutputStream pout = new PacketOutputStream( dout );
        pout.writePacket( packet );
        pout.close();
        dout.close();
        return new PacketByteArray( out.toByteArray() );
    }

    /**
     * A real (though very simple) command line PGP conventional encryptor!
     */
    public static void main( String[] arg ) throws IOException, DecryptException, FormatException
    {
        // Uncomment the following line to enable debugging.
        Utility.debug = new PrintWriter( System.out, true );

        // Gather all the details together
        Toolkit toolkit = new Toolkit( arg[0] );
        Engine engine = toolkit.getEngine();

        // Conventionally encrypt it...
        engine = new ConventionalEncryptionEngine( toolkit, engine );

        // Output the file
        toolkit.setOutputStream( ".pgp", (arg.length==1) ? null : arg[1] );
        toolkit.outputStream.write( engine.toPacketByteArray().toByteArray() );

        // All done. Tell the user.
        toolkit.ui.notice( Lang.translate( "Conventionally encrypted file" ) + ": '" + toolkit.destFilename + "'" );
    }

}

/*package-visibility*/ final class ConvEncHashTester implements HashTester
{
    PacketByteArray pba = null;
    ConvEncryptedData packet;

    ConvEncHashTester( ConvEncryptedData packet )
    {
        this.packet = packet;
    }

   	public boolean testHash( byte[] hash )
    {
        try
        {
            IDEA idea = new IDEA( hash );
            CFB cfb = new CFB( idea );

            int blen = idea.blockLength();
            byte[] buffer   = packet.getBuffer();
            byte[] contents = new byte[ buffer.length - 10 ];
            byte[] iv       = new byte[ 10 ];

            //System.arraycopy( buffer, 0, iv, 0, 10 );
            //System.arraycopy( buffer, 10, contents, 0, contents.length );

            cfb.decrypt( buffer, 0, iv, 0, 10 );
            if ((iv[6] != iv[8]) || (iv[7] != iv[9])) return false;

            cfb.decrypt( buffer, 10, contents, 0, contents.length );
            pba = new PacketByteArray( contents );
            return true;
        }
        catch (FormatException e)
        {
            return false;
        }
    }

}

