// This file is currently unlocked (change this line if you lock the file)
//
// Source code version is 2.3.0
//                        (increment this number EVERY time you make a change)
//
// History: (add to this EVERY time you make a change) 
// (insert details of your change immediately below this line)
//
// 2.3.0        16 May 1997     Jill Baker
//      Original Version
//
// If this file is unlocked, or locked by someone other than you, then you
// may download this file for incorporation into your build, however any
// changes you make to your copy of this file will be considered volatile,
// and will not be incorporated into the master build which is maintained
// at this web site.
//
// If you wish to make permanent changes to this file, you must wait until
// it becomes unlocked (if it is not unlocked already), and then lock it
// yourself. Whenever the file is locked by you, you may upload new master
// versions to the master site. Make sure you update the history information
// above. When your changes are complete, you should relinquish the lock to
// make it available for other developers.

/*
 * Copyright (C) 1995, 1996 Systemics Ltd (http://www.systemics.com/)
 * All rights reserved.
 *
 * This library and applications are FREE FOR COMMERCIAL AND NON-COMMERCIAL USE
 * as long as the conditions within the COPYRIGHT file are adhered to.
 *
 */

package cryptix.pgp.engine;

import cryptix.io.Lang;
import cryptix.pgp.*;
import cryptix.pgp.engine.toolkit.*;
import cryptix.pgp.charset.CharSet;
import cryptix.mime.CRLF;
import java.io.FileOutputStream;
import java.io.IOException;

/**
 * @author Jill Baker
 */
public final class LiteralDataEngine extends Engine
{
    private LiteralData packet;

    public LiteralDataEngine( LiteralData packet )
    {
        this.packet = packet;
    }

    public DecryptStep decryptStep( Toolkit toolkit ) throws DecryptException, IOException
    {
        // Get the raw data
        byte[] buffer = packet.getBuffer();

        // Convert to external char set
        if (packet.mode() == Algorithm.LEGACY)
        {
            try
            {
                buffer = CharSet.decode( buffer, toolkit.charset );
            }
            catch (FormatException e)
            {
                toolkit.ui.warning( e.getMessage() );
            }
            buffer = CRLF.decode( buffer );
        }

        // If this is an EYES_ONLY message then view it, else save it.
        if (packet.filename().equals( "_CONSOLE" ))
        {
            boolean display = toolkit.ui.promptYesNo( "This message is marked \"For your eyes only\". Display now?", false );
            if (display)
            {
                toolkit.ui.status( Lang.translate( "Viewing eyes-only file" ));
                toolkit.plaintextViewer.view( buffer );
            }
        }
        else
        {
            if (toolkit.useStoredFilename)
            {
                Utility.dPrintln( "Plaintext will be saved with filename \"" + packet.filename() + "\"" );
                toolkit.destFilename = packet.filename();
                toolkit.outputStream = new FileOutputStream( toolkit.destFilename );
            }
            String msg = Lang.translate( "Saving file '[]'" );
            toolkit.ui.status( Lang.escape( msg, toolkit.destFilename ) );
            toolkit.outputStream.write( packet.getBuffer() );
            toolkit.outputStream.close();

        }
        toolkit.ui.status();

        return new DecryptStep();
    }

    public PacketByteArray toPacketByteArray() throws IOException
    {
        return new PacketByteArray( packet.save() );
    }

}
