// This file is currently unlocked (change this line if you lock the file)
//
// Source code version is 2.3.0
//                        (increment this number EVERY time you make a change)
//
// History: (add to this EVERY time you make a change) 
// (insert details of your change immediately below this line)
//
// 2.3.0        16 May 1997     Jill Baker
//      Original Version
//
// If this file is unlocked, or locked by someone other than you, then you
// may download this file for incorporation into your build, however any
// changes you make to your copy of this file will be considered volatile,
// and will not be incorporated into the master build which is maintained
// at this web site.
//
// If you wish to make permanent changes to this file, you must wait until
// it becomes unlocked (if it is not unlocked already), and then lock it
// yourself. Whenever the file is locked by you, you may upload new master
// versions to the master site. Make sure you update the history information
// above. When your changes are complete, you should relinquish the lock to
// make it available for other developers.

/*
 * Copyright (C) 1995, 1996 Systemics Ltd (http://www.systemics.com/)
 * All rights reserved.
 *
 * This library and applications are FREE FOR COMMERCIAL AND NON-COMMERCIAL USE
 * as long as the conditions within the COPYRIGHT file are adhered to.
 *
 */

package cryptix.pgp.engine;

import cryptix.io.Lang;
import cryptix.pgp.*;
import cryptix.pgp.engine.toolkit.*;
import cryptix.math.BigInteger;
import cryptix.security.IDEA;
import cryptix.security.rsa.PublicKey;
import cryptix.security.rsa.SecretKey;
import cryptix.util.SensitiveByteArray;
import java.io.ByteArrayOutputStream;
import java.io.DataOutputStream;
import java.io.IOException;
import java.io.PrintWriter;
import java.text.DateFormat;
import java.util.Date;
import java.util.Enumeration;
import java.util.Vector;

/**
 * @author Jill Baker
 */
public final class PublicKeyEncryptionEngine extends Engine
{
    private Vector /*of PublicKeyEncrypted objects*/ recipients;
    private ConvEncryptedData packet;

    /**
     * Constuctor which merely puts all the elements together,
     * assuming they are correct.
     */
    public PublicKeyEncryptionEngine( Vector recipients, ConvEncryptedData packet )
    {
        this.recipients = recipients;
        this.packet = packet;
    }

    /**
     * Constructor which encrypts an existing engine.
     * @param toolkit toolkit defining user preferences
     * @engine Engine object to encrypt
     * @return Engine, now encrypted
     */
    public PublicKeyEncryptionEngine( Toolkit toolkit, Engine engine ) throws IOException, FormatException
    {
        this( toolkit, engine, null );
    }

    /**
     * Constructor which encrypts an existing engine.
     * @param toolkit toolkit defining user preferences
     * @engine Engine object to encrypt
     * @recipients Vector of PublicKeys
     * @return Engine, now encrypted
     */
    public PublicKeyEncryptionEngine( Toolkit toolkit, Engine engine, Vector recipients )
        throws IOException, FormatException
    {
        // Determine recipients
        this.recipients = new Vector();
        if (recipients == null)
        {
            recipients = new Vector();
            Vector keyNames = new Vector();
            String[] keyNamesS = toolkit.keyRings.getFullUserNameList();
            for (int i=0; i<keyNamesS.length; i++) keyNames.addElement( keyNamesS[i] );
            String prompt = Lang.translate( "A user ID is required to select the recipient's public key" ) + ".";
            String uid = toolkit.ui.promptFromList( prompt, keyNames, false );
            Utility.dPrintln( "Key selected = " + uid );
            recipients.addElement( toolkit.keyRings.getKey( uid ));
        }

        // Generate a random IDEA key
        byte[] ideaKey = new byte[ IDEA.KEY_LENGTH ];
        toolkit.randomStream.read( ideaKey );
        SensitiveByteArray sessionKey = new SensitiveByteArray( ideaKey, false );

        // Encrypt IDEA key for each recipient
        Enumeration e = recipients.elements();
        while (e.hasMoreElements())
        {
            PublicKey key = (PublicKey) e.nextElement();
            this.recipients.addElement( new PublicKeyEncrypted( key, sessionKey.getData(), toolkit.randomStream ) );
        }

        // Conventionally encrypt the data itself
        packet = new ConvEncryptedData( engine.toPacketByteArray().toByteArray(), sessionKey.getData(), toolkit.randomStream );
        sessionKey.burn();
    }

    public DecryptStep decryptStep( Toolkit toolkit ) throws DecryptException, IOException
    {
        toolkit.ui.status( Lang.translate( "Decrypting public key encrypted data" ));
        KeyGlob keyRings = toolkit.keyRings;
        KeyID keyId = null;
        Passphrase passphrase = null;
        PacketByteArray pba = null;
        Enumeration e1;
        Enumeration e2;
        if (keyRings != null)
        {
            e1 = recipients.elements();
            while (e1.hasMoreElements())
            {
                PublicKeyEncrypted recipient = (PublicKeyEncrypted) e1.nextElement();
                keyId = recipient.secretKeyID();

                KeyRingEntry kre = keyRings.getSecretKeyRingEntry( keyId );
                if (kre != null)
                {
                    BigInteger encKey = recipient.getEncryptionKey();
                    PubKeyEncHashTester test = new PubKeyEncHashTester( packet, keyRings, keyId, encKey );
                    SecretKeyCertificate cert = (SecretKeyCertificate)kre.certificate();
                    String info = Utility.getKeyInfo( kre );

                    // See if we need a passphrase at all
                    if (cert.getAlgorithm() == Algorithm.NONE)
                    {
                        toolkit.ui.notice( info );
                        test.testHash( null );
                        pba = test.pba;
                        return new DecryptStep( test.pba );
                    }

                    // See if we have the passphrase already
                    if (toolkit.findPassphrase( test ))
                    {
                        toolkit.ui.notice( info );
                        return new DecryptStep( test.pba );
                    }

                    // Input Passphrase manually
                    toolkit.ui.notice(
                        Lang.translate( "You will need a passphrase with which to unlock your RSA secret key" )
                        + "\n" + info );

                    passphraseSkip:
                    {
                        for (int i=0; i<3; i++)
                        {
                            SensitiveByteArray passphraseHash = toolkit.ui.promptPassphrase( toolkit.charset );
                            if (passphraseHash == null) break passphraseSkip;
                            if (test.testHash( passphraseHash.getData() ))
                            {
                                pba = test.pba;
                                toolkit.passphraseVector.addElement( new Passphrase( passphraseHash, toolkit.passphraseTimeout ) );
                                return new DecryptStep( test.pba );
                            }
                            passphraseHash.burn();
                        }
                        toolkit.ui.error( Lang.translate( "Bad passphrase" ));
                    }
                }
            }
        }
        toolkit.ui.error( Lang.translate( "You do not have the secret key required to decrypt this message" ));
        return null;
    }

    public PacketByteArray toPacketByteArray() throws IOException
    {
		ByteArrayOutputStream out = new ByteArrayOutputStream();
        DataOutputStream dout = new DataOutputStream( out );
        PacketOutputStream pout = new PacketOutputStream( dout );
        Enumeration e1 = recipients.elements();
        while (e1.hasMoreElements())
        {
		    pout.writePacket( (PublicKeyEncrypted) e1.nextElement() );
        }
        pout.writePacket( packet );
        pout.close();
        dout.close();
		return new PacketByteArray( out.toByteArray() );
    }

    /**
     * A real (though very simple) command line PGP public key encryptor!
     */
    public static void main( String[] arg ) throws IOException, DecryptException, FormatException
    {
        // Uncomment the following line to enable debugging.
        Utility.debug = new PrintWriter( System.out, true );

        // Gather all the details together
        Toolkit toolkit = new Toolkit( arg[0] );
        Engine engine = toolkit.getEngine();

        // Encrypt it...
        engine = new PublicKeyEncryptionEngine( toolkit, engine );

        // Output the file
        toolkit.setOutputStream( ".pgp", (arg.length==1) ? null : arg[1] );
        toolkit.outputStream.write( engine.toPacketByteArray().toByteArray() );

        // All done. Tell the user.
        toolkit.ui.notice( Lang.translate( "Encrypted file" ) + ": '" + toolkit.destFilename + "'" );
    }

}


/*package-visibility*/ final class PubKeyEncHashTester implements HashTester
{
    KeyGlob keyRings;
    KeyID keyId;
    BigInteger encKey;
    ConvEncryptedData packet;
    PacketByteArray pba = null;

    PubKeyEncHashTester( ConvEncryptedData packet, KeyGlob keyRings, KeyID keyId, BigInteger encKey )
    {
        this.packet = packet;
        this.keyRings = keyRings;
        this.keyId = keyId;
        this.encKey = encKey;
    }

   	public boolean testHash( byte[] hash )
    {
        try
        {
            SecretKey sk;
            if (hash == null)
                sk = keyRings.getSecretKey( keyId );
            else
                sk = keyRings.getSecretKey( keyId, hash );

            Utility.dPrintln( "Secret Key Found" );
            BigInteger convKeyInt = sk.decrypt( encKey );
            Utility.dPrintln( "Conventional Key Decrypted to BigInteger" );

            byte[] convKey = HashFactory.hashFromBigInt( convKeyInt );
            if (convKey == null) return false;
            Utility.dPrintln( "Conventional Key Converted to hash array" );

            Utility.dPrintln( "Going to decrypt Conventionally encrypted data packet" );
            byte contents[] = packet.data( convKey );
            Utility.dPrintln( "Packet decrypted. Now decrypting contents..." );
            pba = new PacketByteArray( contents );
            return true;
        }
        catch (Exception e)
        {
            return false;
        }
    }
}

