// This file is currently unlocked (change this line if you lock the file)
//
// Source code version is 2.5.1
//                        (increment this number EVERY time you make a change)
//
// History: (add to this EVERY time you make a change) 
// (insert details of your change immediately below this line)
//
// 3.0.0        15 May 1997     Systemics
//      Added Java code.  Links not considered/tested.
//
// 2.5.1        15 Apr 1997     Jill Baker
//      Moved this file here from old namespace.
//
// 2.5.0        24 Mar 1997     Original Author not stated
//      Original version
//
/*
 * Copyright (c) 1996, 1997 Systemics Ltd
 * on behalf of the Cryptix Development Team.  All rights reserved.
 */

package cryptix.security;

/**
 * This class implements the Blowfish block cipher in Java.
 *
 * <P>
 * Linked in libraries are not currently tested.  Self_test data
 * is of unknown source.
 *
 * <P>
 * <b>References</b>
 *
 * <P>
 * Blowfish was designed by Bruce Schneier.
 * See
 *      <a href="mailto:schneier@counterpane.com">Bruce Schneier</a>,
 *      "Chapter 14.3 Blowfish,"
 *      <cite>Applied Cryptography</cite>,
 *      Wiley 2nd Ed, 1996,
 * and
 *      <a href="mailto:schneier@counterpane.com">Bruce Schneier</a>,
 *      "Description of a New Variable-Length Key, 64-Bit Cipher (Blowfish),"
 *      <cite>Fast Software Encryption Cambridge Security Workshop Proceedings</cite>,
 *      Springer-Verlag, 1004, pp 191-204.
 *
 * <P>
 * The algorithm is in the public domain.  This implementation was
 * derived from the book and code by Bryan Olsen and Bruce Schneier.
 *
 * <P>
 * <b>Copyright</b> &#169 1996, 1997
 * <a href="http://www.systemics.com/">Systemics Ltd</a>
 * on behalf of the
 * <a href="http://www.systemics.com/docs/cryptix/">
 * Cryptix Development Team</a>.
 * All rights reserved.
 *
 * <p>
 * @author Systemics Ltd
 * @see BlockCipher
 */
public final class Blowfish extends BlockCipher
{
	private static final int REQUIRED_LIB_MAJOR_VER = 1;
	private static final int REQUIRED_LIB_MINOR_VER = 11;
	
	private static final String LIBRARY_NAME = "blowfish";

	private static boolean native_link_ok = false;
	private static boolean native_lib_loaded = false;
	private static String native_link_err = "Class never loaded";

	static 
	{
		// load the DLL or shared library that contains the native code
		// implementation of the Blowfish block cipher.
		try
		{
			System.loadLibrary( LIBRARY_NAME );
			native_lib_loaded = true;
			try
			{
				if ( ( getLibMajorVersion() != REQUIRED_LIB_MAJOR_VER ) || 
					( getLibMinorVersion() < REQUIRED_LIB_MINOR_VER ) )
				{
					native_link_err = "The " + LIBRARY_NAME + " native library " + 
										getLibMajorVersion() + "." + getLibMinorVersion() +
										" is too old to use. Version " + 
										REQUIRED_LIB_MAJOR_VER + "." + REQUIRED_LIB_MINOR_VER + 
										" required.";
				}
				else
				{
					native_link_ok = true;
					native_link_err = null;
				}
			}
			catch ( UnsatisfiedLinkError ule )
			{
				native_link_err = "The " + LIBRARY_NAME + " native library is too old to use. Version " + 
									REQUIRED_LIB_MAJOR_VER + "." + REQUIRED_LIB_MINOR_VER + 
									" required.";
			}
		}
		catch ( UnsatisfiedLinkError ule )
		{
			native_link_err = "The " + LIBRARY_NAME + " native library was not found.";
		}
	}

	public final static boolean
	hasFileLibraryLoaded()
	{
		return native_lib_loaded;
	}

	public final static boolean
	isLibraryCorrect()
	{
		return native_link_ok;
	}

	public final static String
	getLinkErrorString()
	{
		return native_link_err;
	}
	


	//
	// Algorithm Paramaters
	//
	/**
	 * This is the length of a block.
	 */
	public static final int BLOCK_LENGTH = 8;
	public static final int BLOCK_LENGTH_MIN = 8;
	public static final int BLOCK_LENGTH_MAX = 8;

	public static final int KEY_LENGTH = 16;     // recommended not fixed
	public static final int KEY_LENGTH_MAX = 56; // bytes *not* bits
	/** set to one for tests */
	public static final int KEY_LENGTH_MIN = 1;  // a guess, not "correct"
	/** length can be any number of bytes */
	public static final int KEY_LENGTH_MOD = 1;  // length is any string

	/**
	 * A 16 round cipher.  The commentary suggests this could be
	 * variable, although nothing is made of this as a feature.
	 */
	public static final int NUM_ROUNDS = 16;

	/**
	 * Length of the internal buffer for the native code.
	 * Not currently used for the Java code.
	 */
	private static final int INTERNAL_KEY_LENGTH = 4240;

	/**
	 * Internal buffer with the expanded encrypt key.
	 */
	private byte ks[] = null;
 
	private int keyLength;

	/**
	 * @return the block length of this cipher in bytes.
	 */
	public int blockLength() { return BLOCK_LENGTH; }

	/**
	 * @return the length of the current key in bytes.
	 */
	public int keyLength() { return keyLength; }




	/**
	 * Creates a Blowfish block cipher from a given key.
	 * The key length can vary up to 56 bytes.
	 * @param userKey    the user key.
	 */
	public Blowfish(byte userKey[]) 
	{
		setKey(userKey);
	}

	/**
	 * Do nothing constructor - this code is moving in the direction
	 * of the security API.  Not tested yet.
	 */
	public Blowfish() 
	{
	}

	public void
	setKey( byte userKey[] )
	{
//		if ( !native_link_ok )
//		       throw new UnsatisfiedLinkError( native_link_err );

		if (ks != null)
		       CryptoError( "setKey called twice?" );

		resetKey( userKey );
	}

	public void
	resetKey(byte userKey[])
	{

		keyLength = userKey.length;
		if ( userKey.length < KEY_LENGTH_MIN ||
		     userKey.length > KEY_LENGTH_MAX )
			CryptoError( "Blowfish: User key length wrong" );

		if (native_link_ok)
		{
			//
			// is this a magic keyspace available to native code?
			// clean old key?
			ks = new byte[INTERNAL_KEY_LENGTH];

			//
			// Check key expansion for something?
			//
			if (0 == set_ks(userKey))
			{
				int i;
				for (i = 0; i < INTERNAL_KEY_LENGTH; i++)
					if (ks[i] == 0)
						break;
//
// what's going on here?  deprecated API why check zeroes?
//
				CryptoError( "Blowfish: key expansion Error - " + new String( ks, i ) );
			}
		}
		else
			java_set_ks(userKey);

	}

	/**
	 * Encrypt a block.
	 * The in and out buffers can be the same.
	 * @param in The data to be encrypted.
	 * @param in_offset   The start of data within the in buffer.
	 * @param out The encrypted data.
	 * @param out_offset  The start of data within the out buffer.
	 */
	protected void 
	blockEncrypt( byte in[], int in_offset, byte out[], int out_offset )
	{
	    if (native_link_ok ? (null == ks) : (null == p))
		CryptoError( "Blowfish: User key not set" );

	    if (native_link_ok )
		    do_blowfish(in, in_offset, out, out_offset, true);
	    else
		    java_blowfish(in, in_offset, out, out_offset, true);
	}

	/**
	 * Decrypt a block.
	 * The in and out buffers can be the same.
	 * @param in The data to be decrypted.
	 * @param in_offset   The start of data within the in buffer.
	 * @param out The decrypted data.
	 * @param out_offset  The start of data within the out buffer.
	 */
	protected void
	blockDecrypt( byte in[], int in_offset, byte out[], int out_offset )
	{
	    if (native_link_ok ? (null == ks) : (null == p))
		CryptoError( "Blowfish: User key not set." );

//// System.out.println(" " + in[0] + " " + in[1] + " " + in[2] + " " + in[3]);
	    if (native_link_ok )
		    do_blowfish(in, in_offset, out, out_offset, false);
	    else
		    java_blowfish(in, in_offset, out, out_offset, false);
	}


	// The native functions that get the library version.

	private native static int getLibMajorVersion();
	private native static int getLibMinorVersion();

	// these are the native functions that implement the Blowfish algorithm.

	/**
	 * This function expands the user key to an internal form.
	 */
	private native int set_ks( byte userKey[] );

	/**
	 * Encrypt/decrypt the block (depending upon encrypt flag)
	 */
	private native void do_blowfish( byte in[], int in_offset, byte out[], int out_offset, boolean encrypt );



//
// Blowfish in Java.
// Code derived from Bruce Schneier, Bryan Olson and A.M. Kuchling.
//
// Is this a hack?  rather than pass blocks around,
// use a local copy of the two block halfs.  It is, after all, a class.
//
private int left, right;

private int[][] p;
private int[][] sbox;



//
// Now, as we are likely to link in to C code, does it make
// sense to have one copy of this?
// Casts are there becuase thes full-bit ints go negative otherwise.
//

static final int p_init[] = // NUM_ROUNDS + 2
{
 (int) 608135816L, (int)2242054355L, (int) 320440878L, (int)  57701188L,
 (int)2752067618L, (int) 698298832L, (int) 137296536L, (int)3964562569L,
 (int)1160258022L, (int) 953160567L, (int)3193202383L, (int) 887688300L,
 (int)3232508343L, (int)3380367581L, (int)1065670069L, (int)3041331479L,
 (int)2450970073L, (int)2306472731L,
} ;

static final int s_init[][] = // 4 256
{

 { 
 (int)3509652390L, (int)2564797868L, (int) 805139163L, (int)3491422135L,
 (int)3101798381L, (int)1780907670L, (int)3128725573L, (int)4046225305L,
 (int) 614570311L, (int)3012652279L, (int) 134345442L, (int)2240740374L,
 (int)1667834072L, (int)1901547113L, (int)2757295779L, (int)4103290238L,
 (int) 227898511L, (int)1921955416L, (int)1904987480L, (int)2182433518L,
 (int)2069144605L, (int)3260701109L, (int)2620446009L, (int) 720527379L,
 (int)3318853667L, (int) 677414384L, (int)3393288472L, (int)3101374703L,
 (int)2390351024L, (int)1614419982L, (int)1822297739L, (int)2954791486L,
 (int)3608508353L, (int)3174124327L, (int)2024746970L, (int)1432378464L,
 (int)3864339955L, (int)2857741204L, (int)1464375394L, (int)1676153920L,
 (int)1439316330L, (int) 715854006L, (int)3033291828L, (int) 289532110L,
 (int)2706671279L, (int)2087905683L, (int)3018724369L, (int)1668267050L,
 (int) 732546397L, (int)1947742710L, (int)3462151702L, (int)2609353502L,
 (int)2950085171L, (int)1814351708L, (int)2050118529L, (int) 680887927L,
 (int) 999245976L, (int)1800124847L, (int)3300911131L, (int)1713906067L,
 (int)1641548236L, (int)4213287313L, (int)1216130144L, (int)1575780402L,
 (int)4018429277L, (int)3917837745L, (int)3693486850L, (int)3949271944L,
 (int) 596196993L, (int)3549867205L, (int) 258830323L, (int)2213823033L,
 (int) 772490370L, (int)2760122372L, (int)1774776394L, (int)2652871518L,
 (int) 566650946L, (int)4142492826L, (int)1728879713L, (int)2882767088L,
 (int)1783734482L, (int)3629395816L, (int)2517608232L, (int)2874225571L,
 (int)1861159788L, (int) 326777828L, (int)3124490320L, (int)2130389656L,
 (int)2716951837L, (int) 967770486L, (int)1724537150L, (int)2185432712L,
 (int)2364442137L, (int)1164943284L, (int)2105845187L, (int) 998989502L,
 (int)3765401048L, (int)2244026483L, (int)1075463327L, (int)1455516326L,
 (int)1322494562L, (int) 910128902L, (int) 469688178L, (int)1117454909L,
 (int) 936433444L, (int)3490320968L, (int)3675253459L, (int)1240580251L,
 (int) 122909385L, (int)2157517691L, (int) 634681816L, (int)4142456567L,
 (int)3825094682L, (int)3061402683L, (int)2540495037L, (int)  79693498L,
 (int)3249098678L, (int)1084186820L, (int)1583128258L, (int) 426386531L,
 (int)1761308591L, (int)1047286709L, (int) 322548459L, (int) 995290223L,
 (int)1845252383L, (int)2603652396L, (int)3431023940L, (int)2942221577L,
 (int)3202600964L, (int)3727903485L, (int)1712269319L, (int) 422464435L,
 (int)3234572375L, (int)1170764815L, (int)3523960633L, (int)3117677531L,
 (int)1434042557L, (int) 442511882L, (int)3600875718L, (int)1076654713L,
 (int)1738483198L, (int)4213154764L, (int)2393238008L, (int)3677496056L,
 (int)1014306527L, (int)4251020053L, (int) 793779912L, (int)2902807211L,
 (int) 842905082L, (int)4246964064L, (int)1395751752L, (int)1040244610L,
 (int)2656851899L, (int)3396308128L, (int) 445077038L, (int)3742853595L,
 (int)3577915638L, (int) 679411651L, (int)2892444358L, (int)2354009459L,
 (int)1767581616L, (int)3150600392L, (int)3791627101L, (int)3102740896L,
 (int) 284835224L, (int)4246832056L, (int)1258075500L, (int) 768725851L,
 (int)2589189241L, (int)3069724005L, (int)3532540348L, (int)1274779536L,
 (int)3789419226L, (int)2764799539L, (int)1660621633L, (int)3471099624L,
 (int)4011903706L, (int) 913787905L, (int)3497959166L, (int) 737222580L,
 (int)2514213453L, (int)2928710040L, (int)3937242737L, (int)1804850592L,
 (int)3499020752L, (int)2949064160L, (int)2386320175L, (int)2390070455L,
 (int)2415321851L, (int)4061277028L, (int)2290661394L, (int)2416832540L,
 (int)1336762016L, (int)1754252060L, (int)3520065937L, (int)3014181293L,
 (int) 791618072L, (int)3188594551L, (int)3933548030L, (int)2332172193L,
 (int)3852520463L, (int)3043980520L, (int) 413987798L, (int)3465142937L,
 (int)3030929376L, (int)4245938359L, (int)2093235073L, (int)3534596313L,
 (int) 375366246L, (int)2157278981L, (int)2479649556L, (int) 555357303L,
 (int)3870105701L, (int)2008414854L, (int)3344188149L, (int)4221384143L,
 (int)3956125452L, (int)2067696032L, (int)3594591187L, (int)2921233993L,
 (int)   2428461L, (int) 544322398L, (int) 577241275L, (int)1471733935L,
 (int) 610547355L, (int)4027169054L, (int)1432588573L, (int)1507829418L,
 (int)2025931657L, (int)3646575487L, (int) 545086370L, (int)  48609733L,
 (int)2200306550L, (int)1653985193L, (int) 298326376L, (int)1316178497L,
 (int)3007786442L, (int)2064951626L, (int) 458293330L, (int)2589141269L,
 (int)3591329599L, (int)3164325604L, (int) 727753846L, (int)2179363840L,
 (int) 146436021L, (int)1461446943L, (int)4069977195L, (int) 705550613L,
 (int)3059967265L, (int)3887724982L, (int)4281599278L, (int)3313849956L,
 (int)1404054877L, (int)2845806497L, (int) 146425753L, (int)1854211946L,
 }, 

 { 
 (int)1266315497L, (int)3048417604L, (int)3681880366L, (int)3289982499L,
 (int)2909710000L, (int)1235738493L, (int)2632868024L, (int)2414719590L,
 (int)3970600049L, (int)1771706367L, (int)1449415276L, (int)3266420449L,
 (int) 422970021L, (int)1963543593L, (int)2690192192L, (int)3826793022L,
 (int)1062508698L, (int)1531092325L, (int)1804592342L, (int)2583117782L,
 (int)2714934279L, (int)4024971509L, (int)1294809318L, (int)4028980673L,
 (int)1289560198L, (int)2221992742L, (int)1669523910L, (int)  35572830L,
 (int) 157838143L, (int)1052438473L, (int)1016535060L, (int)1802137761L,
 (int)1753167236L, (int)1386275462L, (int)3080475397L, (int)2857371447L,
 (int)1040679964L, (int)2145300060L, (int)2390574316L, (int)1461121720L,
 (int)2956646967L, (int)4031777805L, (int)4028374788L, (int)  33600511L,
 (int)2920084762L, (int)1018524850L, (int) 629373528L, (int)3691585981L,
 (int)3515945977L, (int)2091462646L, (int)2486323059L, (int) 586499841L,
 (int) 988145025L, (int) 935516892L, (int)3367335476L, (int)2599673255L,
 (int)2839830854L, (int) 265290510L, (int)3972581182L, (int)2759138881L,
 (int)3795373465L, (int)1005194799L, (int) 847297441L, (int) 406762289L,
 (int)1314163512L, (int)1332590856L, (int)1866599683L, (int)4127851711L,
 (int) 750260880L, (int) 613907577L, (int)1450815602L, (int)3165620655L,
 (int)3734664991L, (int)3650291728L, (int)3012275730L, (int)3704569646L,
 (int)1427272223L, (int) 778793252L, (int)1343938022L, (int)2676280711L,
 (int)2052605720L, (int)1946737175L, (int)3164576444L, (int)3914038668L,
 (int)3967478842L, (int)3682934266L, (int)1661551462L, (int)3294938066L,
 (int)4011595847L, (int) 840292616L, (int)3712170807L, (int) 616741398L,
 (int) 312560963L, (int) 711312465L, (int)1351876610L, (int) 322626781L,
 (int)1910503582L, (int) 271666773L, (int)2175563734L, (int)1594956187L,
 (int)  70604529L, (int)3617834859L, (int)1007753275L, (int)1495573769L,
 (int)4069517037L, (int)2549218298L, (int)2663038764L, (int) 504708206L,
 (int)2263041392L, (int)3941167025L, (int)2249088522L, (int)1514023603L,
 (int)1998579484L, (int)1312622330L, (int) 694541497L, (int)2582060303L,
 (int)2151582166L, (int)1382467621L, (int) 776784248L, (int)2618340202L,
 (int)3323268794L, (int)2497899128L, (int)2784771155L, (int) 503983604L,
 (int)4076293799L, (int) 907881277L, (int) 423175695L, (int) 432175456L,
 (int)1378068232L, (int)4145222326L, (int)3954048622L, (int)3938656102L,
 (int)3820766613L, (int)2793130115L, (int)2977904593L, (int)  26017576L,
 (int)3274890735L, (int)3194772133L, (int)1700274565L, (int)1756076034L,
 (int)4006520079L, (int)3677328699L, (int) 720338349L, (int)1533947780L,
 (int) 354530856L, (int) 688349552L, (int)3973924725L, (int)1637815568L,
 (int) 332179504L, (int)3949051286L, (int)  53804574L, (int)2852348879L,
 (int)3044236432L, (int)1282449977L, (int)3583942155L, (int)3416972820L,
 (int)4006381244L, (int)1617046695L, (int)2628476075L, (int)3002303598L,
 (int)1686838959L, (int) 431878346L, (int)2686675385L, (int)1700445008L,
 (int)1080580658L, (int)1009431731L, (int) 832498133L, (int)3223435511L,
 (int)2605976345L, (int)2271191193L, (int)2516031870L, (int)1648197032L,
 (int)4164389018L, (int)2548247927L, (int) 300782431L, (int) 375919233L,
 (int) 238389289L, (int)3353747414L, (int)2531188641L, (int)2019080857L,
 (int)1475708069L, (int) 455242339L, (int)2609103871L, (int) 448939670L,
 (int)3451063019L, (int)1395535956L, (int)2413381860L, (int)1841049896L,
 (int)1491858159L, (int) 885456874L, (int)4264095073L, (int)4001119347L,
 (int)1565136089L, (int)3898914787L, (int)1108368660L, (int) 540939232L,
 (int)1173283510L, (int)2745871338L, (int)3681308437L, (int)4207628240L,
 (int)3343053890L, (int)4016749493L, (int)1699691293L, (int)1103962373L,
 (int)3625875870L, (int)2256883143L, (int)3830138730L, (int)1031889488L,
 (int)3479347698L, (int)1535977030L, (int)4236805024L, (int)3251091107L,
 (int)2132092099L, (int)1774941330L, (int)1199868427L, (int)1452454533L,
 (int) 157007616L, (int)2904115357L, (int) 342012276L, (int) 595725824L,
 (int)1480756522L, (int) 206960106L, (int) 497939518L, (int) 591360097L,
 (int) 863170706L, (int)2375253569L, (int)3596610801L, (int)1814182875L,
 (int)2094937945L, (int)3421402208L, (int)1082520231L, (int)3463918190L,
 (int)2785509508L, (int) 435703966L, (int)3908032597L, (int)1641649973L,
 (int)2842273706L, (int)3305899714L, (int)1510255612L, (int)2148256476L,
 (int)2655287854L, (int)3276092548L, (int)4258621189L, (int) 236887753L,
 (int)3681803219L, (int) 274041037L, (int)1734335097L, (int)3815195456L,
 (int)3317970021L, (int)1899903192L, (int)1026095262L, (int)4050517792L,
 (int) 356393447L, (int)2410691914L, (int)3873677099L, (int)3682840055L,
 }, 

 { 
 (int)3913112168L, (int)2491498743L, (int)4132185628L, (int)2489919796L,
 (int)1091903735L, (int)1979897079L, (int)3170134830L, (int)3567386728L,
 (int)3557303409L, (int) 857797738L, (int)1136121015L, (int)1342202287L,
 (int) 507115054L, (int)2535736646L, (int) 337727348L, (int)3213592640L,
 (int)1301675037L, (int)2528481711L, (int)1895095763L, (int)1721773893L,
 (int)3216771564L, (int)  62756741L, (int)2142006736L, (int) 835421444L,
 (int)2531993523L, (int)1442658625L, (int)3659876326L, (int)2882144922L,
 (int) 676362277L, (int)1392781812L, (int) 170690266L, (int)3921047035L,
 (int)1759253602L, (int)3611846912L, (int)1745797284L, (int) 664899054L,
 (int)1329594018L, (int)3901205900L, (int)3045908486L, (int)2062866102L,
 (int)2865634940L, (int)3543621612L, (int)3464012697L, (int)1080764994L,
 (int) 553557557L, (int)3656615353L, (int)3996768171L, (int) 991055499L,
 (int) 499776247L, (int)1265440854L, (int) 648242737L, (int)3940784050L,
 (int) 980351604L, (int)3713745714L, (int)1749149687L, (int)3396870395L,
 (int)4211799374L, (int)3640570775L, (int)1161844396L, (int)3125318951L,
 (int)1431517754L, (int) 545492359L, (int)4268468663L, (int)3499529547L,
 (int)1437099964L, (int)2702547544L, (int)3433638243L, (int)2581715763L,
 (int)2787789398L, (int)1060185593L, (int)1593081372L, (int)2418618748L,
 (int)4260947970L, (int)  69676912L, (int)2159744348L, (int)  86519011L,
 (int)2512459080L, (int)3838209314L, (int)1220612927L, (int)3339683548L,
 (int) 133810670L, (int)1090789135L, (int)1078426020L, (int)1569222167L,
 (int) 845107691L, (int)3583754449L, (int)4072456591L, (int)1091646820L,
 (int) 628848692L, (int)1613405280L, (int)3757631651L, (int) 526609435L,
 (int) 236106946L, (int)  48312990L, (int)2942717905L, (int)3402727701L,
 (int)1797494240L, (int) 859738849L, (int) 992217954L, (int)4005476642L,
 (int)2243076622L, (int)3870952857L, (int)3732016268L, (int) 765654824L,
 (int)3490871365L, (int)2511836413L, (int)1685915746L, (int)3888969200L,
 (int)1414112111L, (int)2273134842L, (int)3281911079L, (int)4080962846L,
 (int) 172450625L, (int)2569994100L, (int) 980381355L, (int)4109958455L,
 (int)2819808352L, (int)2716589560L, (int)2568741196L, (int)3681446669L,
 (int)3329971472L, (int)1835478071L, (int) 660984891L, (int)3704678404L,
 (int)4045999559L, (int)3422617507L, (int)3040415634L, (int)1762651403L,
 (int)1719377915L, (int)3470491036L, (int)2693910283L, (int)3642056355L,
 (int)3138596744L, (int)1364962596L, (int)2073328063L, (int)1983633131L,
 (int) 926494387L, (int)3423689081L, (int)2150032023L, (int)4096667949L,
 (int)1749200295L, (int)3328846651L, (int) 309677260L, (int)2016342300L,
 (int)1779581495L, (int)3079819751L, (int) 111262694L, (int)1274766160L,
 (int) 443224088L, (int) 298511866L, (int)1025883608L, (int)3806446537L,
 (int)1145181785L, (int) 168956806L, (int)3641502830L, (int)3584813610L,
 (int)1689216846L, (int)3666258015L, (int)3200248200L, (int)1692713982L,
 (int)2646376535L, (int)4042768518L, (int)1618508792L, (int)1610833997L,
 (int)3523052358L, (int)4130873264L, (int)2001055236L, (int)3610705100L,
 (int)2202168115L, (int)4028541809L, (int)2961195399L, (int)1006657119L,
 (int)2006996926L, (int)3186142756L, (int)1430667929L, (int)3210227297L,
 (int)1314452623L, (int)4074634658L, (int)4101304120L, (int)2273951170L,
 (int)1399257539L, (int)3367210612L, (int)3027628629L, (int)1190975929L,
 (int)2062231137L, (int)2333990788L, (int)2221543033L, (int)2438960610L,
 (int)1181637006L, (int) 548689776L, (int)2362791313L, (int)3372408396L,
 (int)3104550113L, (int)3145860560L, (int) 296247880L, (int)1970579870L,
 (int)3078560182L, (int)3769228297L, (int)1714227617L, (int)3291629107L,
 (int)3898220290L, (int) 166772364L, (int)1251581989L, (int) 493813264L,
 (int) 448347421L, (int) 195405023L, (int)2709975567L, (int) 677966185L,
 (int)3703036547L, (int)1463355134L, (int)2715995803L, (int)1338867538L,
 (int)1343315457L, (int)2802222074L, (int)2684532164L, (int) 233230375L,
 (int)2599980071L, (int)2000651841L, (int)3277868038L, (int)1638401717L,
 (int)4028070440L, (int)3237316320L, (int)   6314154L, (int) 819756386L,
 (int) 300326615L, (int) 590932579L, (int)1405279636L, (int)3267499572L,
 (int)3150704214L, (int)2428286686L, (int)3959192993L, (int)3461946742L,
 (int)1862657033L, (int)1266418056L, (int) 963775037L, (int)2089974820L,
 (int)2263052895L, (int)1917689273L, (int) 448879540L, (int)3550394620L,
 (int)3981727096L, (int) 150775221L, (int)3627908307L, (int)1303187396L,
 (int) 508620638L, (int)2975983352L, (int)2726630617L, (int)1817252668L,
 (int)1876281319L, (int)1457606340L, (int) 908771278L, (int)3720792119L,
 (int)3617206836L, (int)2455994898L, (int)1729034894L, (int)1080033504L,
 }, 

 { 
 (int) 976866871L, (int)3556439503L, (int)2881648439L, (int)1522871579L,
 (int)1555064734L, (int)1336096578L, (int)3548522304L, (int)2579274686L,
 (int)3574697629L, (int)3205460757L, (int)3593280638L, (int)3338716283L,
 (int)3079412587L, (int) 564236357L, (int)2993598910L, (int)1781952180L,
 (int)1464380207L, (int)3163844217L, (int)3332601554L, (int)1699332808L,
 (int)1393555694L, (int)1183702653L, (int)3581086237L, (int)1288719814L,
 (int) 691649499L, (int)2847557200L, (int)2895455976L, (int)3193889540L,
 (int)2717570544L, (int)1781354906L, (int)1676643554L, (int)2592534050L,
 (int)3230253752L, (int)1126444790L, (int)2770207658L, (int)2633158820L,
 (int)2210423226L, (int)2615765581L, (int)2414155088L, (int)3127139286L,
 (int) 673620729L, (int)2805611233L, (int)1269405062L, (int)4015350505L,
 (int)3341807571L, (int)4149409754L, (int)1057255273L, (int)2012875353L,
 (int)2162469141L, (int)2276492801L, (int)2601117357L, (int) 993977747L,
 (int)3918593370L, (int)2654263191L, (int) 753973209L, (int)  36408145L,
 (int)2530585658L, (int)  25011837L, (int)3520020182L, (int)2088578344L,
 (int) 530523599L, (int)2918365339L, (int)1524020338L, (int)1518925132L,
 (int)3760827505L, (int)3759777254L, (int)1202760957L, (int)3985898139L,
 (int)3906192525L, (int) 674977740L, (int)4174734889L, (int)2031300136L,
 (int)2019492241L, (int)3983892565L, (int)4153806404L, (int)3822280332L,
 (int) 352677332L, (int)2297720250L, (int)  60907813L, (int)  90501309L,
 (int)3286998549L, (int)1016092578L, (int)2535922412L, (int)2839152426L,
 (int) 457141659L, (int) 509813237L, (int)4120667899L, (int) 652014361L,
 (int)1966332200L, (int)2975202805L, (int)  55981186L, (int)2327461051L,
 (int) 676427537L, (int)3255491064L, (int)2882294119L, (int)3433927263L,
 (int)1307055953L, (int) 942726286L, (int) 933058658L, (int)2468411793L,
 (int)3933900994L, (int)4215176142L, (int)1361170020L, (int)2001714738L,
 (int)2830558078L, (int)3274259782L, (int)1222529897L, (int)1679025792L,
 (int)2729314320L, (int)3714953764L, (int)1770335741L, (int) 151462246L,
 (int)3013232138L, (int)1682292957L, (int)1483529935L, (int) 471910574L,
 (int)1539241949L, (int) 458788160L, (int)3436315007L, (int)1807016891L,
 (int)3718408830L, (int) 978976581L, (int)1043663428L, (int)3165965781L,
 (int)1927990952L, (int)4200891579L, (int)2372276910L, (int)3208408903L,
 (int)3533431907L, (int)1412390302L, (int)2931980059L, (int)4132332400L,
 (int)1947078029L, (int)3881505623L, (int)4168226417L, (int)2941484381L,
 (int)1077988104L, (int)1320477388L, (int) 886195818L, (int)  18198404L,
 (int)3786409000L, (int)2509781533L, (int) 112762804L, (int)3463356488L,
 (int)1866414978L, (int) 891333506L, (int)  18488651L, (int) 661792760L,
 (int)1628790961L, (int)3885187036L, (int)3141171499L, (int) 876946877L,
 (int)2693282273L, (int)1372485963L, (int) 791857591L, (int)2686433993L,
 (int)3759982718L, (int)3167212022L, (int)3472953795L, (int)2716379847L,
 (int) 445679433L, (int)3561995674L, (int)3504004811L, (int)3574258232L,
 (int)  54117162L, (int)3331405415L, (int)2381918588L, (int)3769707343L,
 (int)4154350007L, (int)1140177722L, (int)4074052095L, (int) 668550556L,
 (int)3214352940L, (int) 367459370L, (int) 261225585L, (int)2610173221L,
 (int)4209349473L, (int)3468074219L, (int)3265815641L, (int) 314222801L,
 (int)3066103646L, (int)3808782860L, (int) 282218597L, (int)3406013506L,
 (int)3773591054L, (int) 379116347L, (int)1285071038L, (int) 846784868L,
 (int)2669647154L, (int)3771962079L, (int)3550491691L, (int)2305946142L,
 (int) 453669953L, (int)1268987020L, (int)3317592352L, (int)3279303384L,
 (int)3744833421L, (int)2610507566L, (int)3859509063L, (int) 266596637L,
 (int)3847019092L, (int) 517658769L, (int)3462560207L, (int)3443424879L,
 (int) 370717030L, (int)4247526661L, (int)2224018117L, (int)4143653529L,
 (int)4112773975L, (int)2788324899L, (int)2477274417L, (int)1456262402L,
 (int)2901442914L, (int)1517677493L, (int)1846949527L, (int)2295493580L,
 (int)3734397586L, (int)2176403920L, (int)1280348187L, (int)1908823572L,
 (int)3871786941L, (int) 846861322L, (int)1172426758L, (int)3287448474L,
 (int)3383383037L, (int)1655181056L, (int)3139813346L, (int) 901632758L,
 (int)1897031941L, (int)2986607138L, (int)3066810236L, (int)3447102507L,
 (int)1393639104L, (int) 373351379L, (int) 950779232L, (int) 625454576L,
 (int)3124240540L, (int)4148612726L, (int)2007998917L, (int) 544563296L,
 (int)2244738638L, (int)2330496472L, (int)2058025392L, (int)1291430526L,
 (int) 424198748L, (int)  50039436L, (int)  29584100L, (int)3605783033L,
 (int)2429876329L, (int)2791104160L, (int)1057563949L, (int)3255363231L,
 (int)3075367218L, (int)3463963227L, (int)1469046755L, (int) 985887462L,
 }, 

}; /* s_init[][] */




public boolean test = false;

private void
crypt_block(boolean encrypt)
{
	//
	// P0 is the encrypt schedule, P1 is decrypt, which is simply the
	// reverse.
	//
	int[] P = encrypt ? p[0] : p[1];

	//
	// left and right are locals already set up with the two halves.
	//
	for (int i = 0; i < NUM_ROUNDS; i++)
	{
		if (test) System.out.println("    l " + left + "  \tr " + right + "  \t Pi " + P[i]);
		left  ^= P[i];
		int ls = left;

		//
		// Function F
		//
		int four  = sbox[3][ls & 0xFF];
		ls >>>= 8;
		int three = sbox[2][ls & 0xFF];
		ls >>>= 8;
		int two   = sbox[1][ls & 0xFF];
		ls >>>= 8;
		int one   = sbox[0][ls & 0xFF];

		right ^= ((one + two) ^ three) + four;

		int t = left;   // swap for next round
		left = right;
		right = t;
	}

	int t = left;   // back out the last swap
	left = right;
	right = t;

	right = right ^ P[NUM_ROUNDS] ;
	left = left ^ P[NUM_ROUNDS+1] ;
}

//
// Do a block encrypt from a buffer, into another buffer.
// Copies the bytes to and from the internal left and right halves.
//
private void
java_blowfish(byte[] in, int in_off, byte[] out, int out_off, boolean encrypt)
{
	int i = in_off;
	left = 0xFF & in[i++];
	left <<= 8;
	left |= 0xFF & in[i++];
	left <<= 8;
	left |= 0xFF & in[i++];
	left <<= 8;
	left |= 0xFF & in[i++];
	if (test) System.out.println("    left " + left);

	right = 0xFF & in[i++];
	right <<= 8;
	right |= 0xFF & in[i++];
	right <<= 8;
	right |= 0xFF & in[i++];
	right <<= 8;
	right |= 0xFF & in[i++];
	if (test) System.out.println("    right " + right);

	// Something wrong with this maths??????
	//	left  = ((0xFF & in[3 + in_off])      ) |
	//  	 	((0xFF & in[2 + in_off]) <<  8) |
	//		((0xFF & in[1 + in_off]) << 16) |
	//		((0xFF & in[0 + in_off]) << 24);
	//	right = (0xFF & (in[7 + in_off])      ) |
	//		(0xFF & (in[6 + in_off]) <<  8) | 
	//		(0xFF & (in[5 + in_off]) << 16) |
	//		(0xFF & (in[4 + in_off]) << 24);

	crypt_block(encrypt) ;

	i = 8 + out_off;
	out[--i] = (byte)(right & 0xff);
	right >>>= 8;
	out[--i] = (byte)(right & 0xff);
	right >>>= 8;
	out[--i] = (byte)(right & 0xff);
	right >>>= 8;
	out[--i] = (byte)(right & 0xff);

	out[--i] = (byte)(left & 0xff);
	left >>>= 8;
	out[--i] = (byte)(left & 0xff);
	left >>>= 8;
	out[--i] = (byte)(left & 0xff);
	left >>>= 8;
	out[--i] = (byte)(left & 0xff);

	left = right = 0;  // cleanup
}

private void CryptoError(String s)
{
	System.out.println("CryptoError: " + s);
	System.exit(1);
}


//
// Set the key schedule.
// Initialise the sboxes and the P boxes from the key and init data.
//
private void
java_set_ks(byte[] key)
{

	p = new int[2][NUM_ROUNDS + 2];
	sbox = new int[4][256];

	int check = 0;

	//
	// Step 1.
	// Copy the initial values (pi) into the P vectors.
	//
	for (int i = 0; i < NUM_ROUNDS + 2; i++)
	{
	    p[0][i] = p_init[i] ;
	    // System.arraycopy(p_init, 0, p[0], 0, p_init.length);
	    p[1][NUM_ROUNDS + 1 - i] = p_init[i];
	    check = ((check << 1) | (check >>> 31)) + p_init[i];
	}


	//
	// Initialise the s-boxes.  Straight copy.
	//
	for (int i = 0; i < 4; i++)
	    for (int j = 0; j < 256; j++)
	    {
		sbox[i][j] = s_init[i][j];
		check = ((check*13) << 11 | (check*13) >>> 21)
			+ s_init[i][j] ;	 
	    }

	//
	// Where did this number come from?
	//
	if (check != 0x55861a61)
		CryptoError( "Skeys failed to initialise?" );


	//
	// Self test.
	// Encrypt 0 ten times.  Check result and decrypt it 10 times.
	//
	left = right = 0;                    // result in shared variables
	int st1 = 0;
	for (st1 = 0; st1 < 10; st1++)
		crypt_block(true);

	if (left != 0xaafe4ebd)
		CryptoError( "crypt_block failed encrypt(0) self test? " + left + " " + right);

	while(--st1 >= 0)
		crypt_block(false);

	if (0 != left || 0 != right)
		CryptoError( "crypt_block failed decrypt() self test? " + left + " " + right);


	
	//
	// 2. Make the key.  Xor the key with all of P.
	//
	int k = 0 ;
	for (int rnd = 0; rnd < NUM_ROUNDS + 2; rnd++) 
	{
		int data = 0;
		for (int sb = 0; sb < 4; sb++) 
			data = (data << 8) | (key[k++ % keyLength] & 0xFF);
		p[0][rnd] ^= data; 
	}


	//
	// 3. Encrypt a block of 0.
	// 4. Put that into P[0] and P[1].
	// 5. Then encrypt result again,
	// 6. Put that new result into P[2], P[3].
	// 7. repeat ...
	//
	for (int rnd = 0; rnd < NUM_ROUNDS + 2; rnd += 2) 
	{
	    crypt_block(true);
	    p[0][rnd] = p[1][NUM_ROUNDS + 1 - rnd] = left;
	    p[0][rnd + 1] = p[1][NUM_ROUNDS - rnd] = right;
	}

	//
	// ... and for all the sboxes as well.
	//
	for (int rnd = 0; rnd < 4; rnd++)
	    for (int sb = 0; sb < 256; sb += 2)
	    {
		crypt_block(true);
		sbox[rnd][sb]   = left;
		sbox[rnd][sb+1] = right;
	    }

	//
	// Self test.
	// Encrypt 0 ten times.  Decrypt it ten times.  Should be the same.
	//
	left = right = 0;                    // result in shared variables
	int st2 = 0;
	for (st2 = 0; st2 < 10; st2++)
		crypt_block(true);

	while(--st2 >= 0)
		crypt_block(false);

	if (0 != left || 0 != right)
		CryptoError( "crypt_block failed self test 2? " +
				left + " " + right);

	if (test) System.out.println("Self Test 2 Good");


	return ;
}



// // // // // // // // //  T E S T  // // // // // // // // // // // // 

//
// Don't expand this code please without thinking about it,
// much better to write a separate program.
//
/**
 * Entry point for very basic <code>self_test</code>.
 */
public static final void
main(String argv[]) 
{
	try { self_test(); }
	catch(Throwable t) { t.printStackTrace(); }
}

//
// This is (apparently) the official certification data.
// Use ints as Java grumbles about negative hex values.
//
static final private byte[][][] tests =
{
  { // cert 1
    { // key
     97, 98, 99, 100, 101, 102, 103, 104, 105, 106, 107, 108, 109, 110,
     111, 112, 113, 114, 115, 116, 117, 118, 119, 120, 121, 122,
    },
    { 66, 76, 79, 87, 70, 73, 83, 72, },   // plain
    { 50, 78, -48, -2, -12, 19, -94, 3, }, // cipher
  },
  { // cert 2
    {
      87, 104, 111, 32, 105, 115, 32, 74, 111, 104, 110, 32, 71, 97, 108,
      116, 63,
    },
    { -2, -36, -70, -104, 118, 84, 50, 16, },
    { -52, -111, 115, 43, -128, 34, -10, -124, },
  },
  { // cert 3 - Ayn Rand with 4th ( ) and 8th (d) chars hi bit set.
    { 65, 121, 110, -96, 82, 97, 110, -28, }, // key
    { -2, -36, -70, -104, 118, 84, 50, 16, }, // plain
    { -31, 19, -12, 16, 44, -4, -50, 67, },   // cipher
  },
};

/**
 * Do some basic tests.
 * Three of the certification data are included only, no output,
 * success or exception.
 * If you want more, write a test program!
 * @see cryptix.examples.TestBlowfish
 */
public static final void
self_test()
	throws CryptoError
{
	for (int i = 0; i < tests.length; i++)
	{
		byte[][] test = tests[i];
		byte[] key    = test[0];
		byte[] plain  = test[1];
		byte[] cipher = test[2];

		Blowfish cryptor = new Blowfish(key);
		byte[] tmp = new byte[cryptor.blockLength()];

		cryptor.encrypt(plain, tmp);
		if (notEquals(cipher, tmp))
			throw new CryptoError("encrypt #"+ i +" failed");
		cryptor.decrypt(cipher, tmp);
		if (notEquals(plain, tmp))
			throw new CryptoError("decrypt #"+ i +" failed");
	}
}

private static final boolean
notEquals(byte[] a, byte[] b)
{
	for (int i = 0; i < a.length; i++)
	{
		if (a[i] != b[i])
			return true;
		// if (a[i] == -4) // force a trip up
		// 	return true;
	}
	return false;
}

} /* class */
