// This file is currently locked by iang
//
// Source code version is 3.0.0
//                        (increment this number EVERY time you make a change)
//
// History: (add to this EVERY time you make a change) 
// (insert details of your change immediately below this line)
//
// 3.0.0        05 May 1997     Systemics
//      Added Geoffrey Keating's Java code to DLL stubs, wrote doco, etc.
//	Moved copyright notices to end.
//
// X.X.X        ?? ??? 1996     Geoffrey Keating
//      Ported Eric Young's C version to Java.
//
// 2.5.1        15 Apr 1997     Jill Baker
//      Moved this file here from old namespace.
//
// 2.5.0        24 Mar 1997     Systemics
//      Original version - DLL links only
//
/*
 * Copyright (c) 1995, 1996, 1997 Systemics Ltd
 * on behalf of the Cryptix Development Team.  All rights reserved.
 */

package cryptix.security;

import java.io.PrintStream;

/**
 * DES is a block cipher with an 8 byte block size.  The key length
 * is 8 bytes, but only 56 bits are used as the parity bit in each
 * byte is ignored.
 * <P>
 * This algorithm has been seriously analysed over the last 30 years,
 * and no significant weaknesses have been reported.  It's only flaw
 * is that the key length of 56 bits makes it relatively easy to
 * brute-force it.  To overcome this near-fatal flaw, it is recommended
 * that DES be used in triple-DES mode.  Need reference here...
 *
 * <P>
 * <b>References</b>
 *
 * <P>
 * DES was written by IBM and first released in 1976.
 * See
 *      <a href="mailto:schneier@counterpane.com">Bruce Schneier</a>,
 *      "Chapter 12 Data Encryption Standard,"
 *      <cite>Applied Cryptography</cite>,
 *      Wiley 2nd Ed, 1996
 * for detailed information.  The algorithm is freely usable.
 *
 * <P>
 * <bold>Copyright</bold> &#169 1995, 1996, 1997
 * <a href="http://www.systemics.com/">Systemics Ltd</a>
 * on behalf of the
 * <a href="http://www.systemics.com/docs/cryptix/">
 * Cryptix Development Team</a>.
 * All rights reserved.
 *
 * <P>
 * @author Geoffrey Keating (this Java implementation)
 * @author Eric Young (SPtrans)
 * @see BlockCipher
 */
public final class DES extends BlockCipher
{
	private static final int REQUIRED_LIB_MAJOR_VER = 1;
	private static final int REQUIRED_LIB_MINOR_VER = 11;
	
	private static final String LIBRARY_NAME = "des";

	private static boolean native_link_ok = false;
	private static boolean native_lib_loaded = false;
	private static String native_link_err = "Class never loaded";

	static 
	{
		// load the DLL or shared library that contains the native code
		// implementation of the des block cipher.
		try
		{
			System.loadLibrary( LIBRARY_NAME );
			native_lib_loaded = true;
			try
			{
				if ( ( getLibMajorVersion() != REQUIRED_LIB_MAJOR_VER ) || 
					( getLibMinorVersion() < REQUIRED_LIB_MINOR_VER ) )
				{
					native_link_err = "The " +
					    LIBRARY_NAME + " native library " + 
					    getLibMajorVersion() + "." +
					    getLibMinorVersion() +
					    " is too old to use. Version " + 
					    REQUIRED_LIB_MAJOR_VER + "." +
					    REQUIRED_LIB_MINOR_VER + 
					    " required.";
				}
				else
				{
					native_link_ok = true;
					native_link_err = null;
				}
			}
			catch ( UnsatisfiedLinkError ule )
			{
				native_link_err = "The " +
				    LIBRARY_NAME +
				    " native library is too old to use." +
				    "Version " +
				    REQUIRED_LIB_MAJOR_VER + "." +
				    REQUIRED_LIB_MINOR_VER +
				    " required.";
			}
		}
		catch ( UnsatisfiedLinkError ule )
		{
			native_link_err = "The " +
			    LIBRARY_NAME + " native library was not found.";
		}
	}

	public final static boolean
	hasFileLibraryLoaded()
	{
		return native_lib_loaded;
	}

	public final static boolean
	isLibraryCorrect()
	{
		return native_link_ok;
	}

	public final static String
	getLinkErrorString()
	{
		return native_link_err;
	}



	/**
	 * The length of a block - DEPRECATED - use blockLength() instead.
	 */
	public static final int BLOCK_LENGTH = 8;

	/**
	 * The length of a the user key - DEPRECATED - use keyLength() instead.
	 */
	public static final int KEY_LENGTH = 8;

	private static final int ROUNDS = 16;

	/* The length in ints of the key after it has been scheduled. */
	static private final int INTERNAL_KEY_LENGTH = ROUNDS*2;
	//
	// This was in the original stub, but appears to be wrong.
	//
	// This is the length of the internal buffer for the native code.
	// private static final int INTERNAL_KEY_LENGTH = 128;

	/**
	 * Return the block length of this cipher.
	 * @return       the block length in bytes is 8
	 */
	public int 
	blockLength()
	{
	    return BLOCK_LENGTH;
	}

	/**
	 * Return the key length for this cipher.
	 * @returns      the key length in bytes is 8
	 */
	public int 
	keyLength()
	{
	    return KEY_LENGTH;
	}



	/**
	 * Create a DES block cipher from a key in a byte array.
	 * The byte array must be keyLength bytes long.
	 * @param userKey    the user key in a byte[ keyLength() ] array
	 * @throws           CryptoError if length of key array is wrong
	 * @throws UnsatisfiedLinkError if the library is not of the correct version
	 */
	public DES( byte userKey[] ) 
	{
		if ( userKey.length != KEY_LENGTH )
			throw new CryptoError( "DES: User key length wrong" );

		// this was in the original stub but appears wrong.
		// ks = new byte[INTERNAL_KEY_LENGTH];

		if (native_link_ok)
			set_ks(userKey);
		else
			java_ks(userKey);
	}

	//
	// Encrypt a block.
	// The in and out buffers can be the same.
	// @param in The data to be encrypted.
	// @param in_offset   The start of data within the in buffer.
	// @param out The encrypted data.
	// @param out_offset  The start of data within the out buffer.
	//
	protected void 
	blockEncrypt( byte in[], int in_offset, byte out[], int out_offset )
	{
		if ( key == null )
			throw new CryptoError( "DES: User key not set" );

		if (native_link_ok)
			do_des( in, in_offset, out, out_offset, true );
		else
			java_encrypt( in, in_offset, out, out_offset, true );
	}

	//
	// Decrypt a block.
	// The in and out buffers can be the same.
	// @param in The data to be decrypted.
	// @param in_offset   The start of data within the in buffer.
	// @param out The decrypted data.
	// @param out_offset  The start of data within the out buffer.
	//
	protected void
	blockDecrypt( byte in[], int in_offset, byte out[], int out_offset )
	{
		if ( key == null )
			throw new CryptoError( "DES: User key not set" );

		if (native_link_ok)
			do_des( in, in_offset, out, out_offset, false );
		else
			java_encrypt( in, in_offset, out, out_offset, false );
	}



	//
	//	The native methods that implement the DES algorithm.
	//	Synchronised for convenience of the .so developer
	//
	//	Place markers, untested, unworking.
	//	For the future: maybe change it to pass the key schedule
	//	in to save the C from allocating its own memory, a la IDEA.
	//
	private native static int getLibMajorVersion();	// get native version
	private native static int getLibMinorVersion();	// get native version

	//
	//	Expand the user key to an internal form.
	//
	private native void set_ks( byte userKey[] );

	//
	// Encrypt/decrypt the block (depending upon encrypt flag)
	//
	private native void do_des( byte in[], int in_offset, byte out[], int out_offset, boolean encrypt );



	public String dump( int depth )
	{
		if (depth <= 0) return "";

		String s = new String("DES: dump");

		s += " " + keyLength() + "/" + blockLength() + "/" + ROUNDS;
		if (--depth <= 0) return s;

		s += " key = \n";
		for (int i = 0; i < key.length; i++)
		{
			if ((i % 8) == 0) s += "\n";
			s += " " + Long.toString(key[i] & 0x00FFFFFFFFL, 16);
		}
		if (--depth <= 0) return s;


		if ((depth -= 10) <= 0) return s; // you really don't want this

		s += "\n skb = \n";
		for (int i = 0; i < skb.length; i++)
		{
			if ((i % 8) == 0) s += "\n";
			s += " " + Long.toString(skb[i] & 0x00FFFFFFFFL, 16);
		}
		if (--depth <= 0) return s;

		s += "\n SPtrans = \n";
		for (int i = 0; i < SPtrans.length; i++)
		{
			if ((i % 8) == 0) s += "\n";
			s += " " + Long.toString(SPtrans[i] & 0x00FFFFFFFFL, 16);
		}
		if (--depth <= 0) return s;

		return s ;
	}
	
/////////////////////////////////// START DES CODE ///////////////



	/* The key after it has been scheduled. */
	private int key[] = new int[INTERNAL_KEY_LENGTH];
	
	/* Table for PC2 permutation in key schedule. */
	static private int skb[] = new int[8*64];
	static  {
		for (int i = 0; i < 8*64; i++)
			skb[i] = 0;

		/* represents the bit number that each permuted bit is derived from */
		String cd = "D]PKESYM`UBJ\\@RXA`I[T`HC`LZQ"+"\\PB]TL`[C`JQ@Y`HSXDUIZRAM`EK";
		int l;
		int numdone = 0;
		int opos = 0;
		for (int c = 0; c < cd.length(); c++)
			if ((l = cd.charAt(c) - '@') != 32)
			{
				int indexbit = 1 << numdone;
				for (int i = 0; i < 64; i++)
					if ((indexbit & i) != 0)
						skb[opos+i] |= 1 << l;
				if (++numdone == 6)
				{
					opos += 64;
					numdone = 0;
				}
			}
	}

	private void java_ks(byte userkey[])
	{
		int c, d;	// These need to be fast

		int t;
		c = userkey[0] & 0xFF | userkey[1] << 8 & 0xFF00 |
			userkey[2] << 16 & 0xFF0000 | userkey[3] << 24 & 0xFF000000;
		d = userkey[4] & 0xFF | userkey[5] << 8 & 0xFF00 |
			userkey[6] << 16 & 0xFF0000 | userkey[7] << 24 & 0xFF000000;

// TEST */ System.out.println("c " + c + "\td " + d);

		/* PC1, as per Eric Young. */
		t = (d>>>4 ^ c) & 0x0f0f0f0f;
		c ^= t;
		d ^= t << 4;
		// System.out.println("c " + c + "\td " + d + "       t:" + t);
		t = ((c<<18) ^ c) & 0xcccc0000;
		// t = (int)(((c<<18) ^ c) & 0xcccc0000L); // jolt guava fix
		// System.out.println(" c " + c + "\td " + d + "       t:" + t);

////t = c<<18;
////t ^= c;
////System.out.println(" c " + c + "\td " + d + "       t:" + t);
////t &= 0xcccc0000;
////System.out.println(" c " + c + "\td " + d + "       t:" + t);

		c ^= t ^ (t>>>18);		
////System.out.println(" c4 " + c + "\td " + d + "       t:" + t);

		t = ((d<<18) ^ d) & 0xcccc0000;
		// t = (int)(((d<<18) ^ d) & 0xcccc0000L); // jolt guava fix
////		System.out.println(" c5 " + c + "\td " + d + "       t:" + t);

////t = d<<18;
////t ^= d;
////System.out.println(" c6 " + c + "\td " + d + "       t:" + t);
////
////int t2;
////t2 = 0xcccc0000L & t;         // <<<<<==== BANG no change
////t2 = (int)(0xcccc0000L & t);  // this works
////System.out.println(" c7 " + c + "\td " + d + "       t:" + t2);

		d ^= t ^ (t>>>18);		
////System.out.println("c " + c + "\td " + d + "       t:" + t2);
		t = (d>>>1 ^ c) & 0x55555555;
		c ^= t;
		d ^= t << 1;
////System.out.println("c " + c + "\td " + d + "       t:" + t);
		t = (c>>>8 ^ d) & 0x00ff00ff;
		d ^= t;
////System.out.println("c " + c + "\td " + d + "       t:" + t);
		c ^= t << 8;
		t = (d>>>1 ^ c) & 0x55555555;
		c ^= t;
		d ^= t << 1;
////System.out.println("c " + c + "\td " + d + "       t:" + t);

		d = d<<16 & 0x00ff0000 | d & 0x0000ff00 | d>>16 & 0xFF | c>>4 & 0x0F000000;
		c &= 0x0fffffff;

////System.out.println("c " + c + "\td " + d + "       t:" + t);
		
		for (int i=0; i<ROUNDS; i++)
		{
			int s;
			
			if ((0x7EFC >> i & 1) == 1)
				{ c = c>>>2 | c<<26; d = d>>>2 | d<<26; }
			else
				{ c = c>>>1 | c<<27; d = d>>>1 | d<<27; }
			c &= 0x0fffffff;
			d &= 0x0fffffff;

			s= skb[        c     &0x3f                            ]|
			   skb[0x040 | c>> 6 &0x03 | c>> 7 &0x3c              ]|
			   skb[0x080 | c>>13 &0x0f | c>>14 &0x30              ]|
			   skb[0x0C0 | c>>20 &0x01 | c>>21 &0x06 | c>>22 &0x38];
			t= skb[0x100 | d     &0x3f              ]|
			   skb[0x140 | d>> 7 &0x03 | d>> 8 &0x3c]|
			   skb[0x180 | d>>15 &0x3f              ]|
			   skb[0x1C0 | d>>21 &0x0f | d>>22 &0x30];

			key[i<<1] = t<<16 | s&0x0000ffff;
			s = s>>>16 | t&0xffff0000;
			key[i<<1|1] = s<<4 | s>>>28;
		}
	}
		

	/* table for S-boxes and permutations, used in encrypt_base. */
	static private int SPtrans[] = new int[8*64];
	static  {
		/* I'd _really_ like to just say 'SPtrans =  { ... }', but
		   that would be terribly inefficient (code size + time). 
		   Instead we use a compressed representation. */
		String spt =
		   "g3H821:80:H03BA0@N1290BAA88::3112aIH8:8282@0@AH0:1W3A8P810@22;22A"+
		   "18^@9H9@129:<8@822`?:@0@8PH2H81A19:G1@03403A0B1;:0@1g192:@919AA0A"+
		   "109:W21492H@0051919811:215011139883942N8::3112A2:31981jM118::A101"+
		   "@I88:1aN0<@030128:X;811`920:;H0310D1033@W980:8A4@804A3803o1A2021B"+
		   "2:@1AH023GA:8:@81@@12092B:098042P@:0:A0HA9>1;289:@1804:40Ph=1:H0I"+
		   "0HP0408024bC9P8@I808A;@0@0PnH0::8:19J@818:@iF0398:8A9H0<13@001@11"+
		   "<8;@82B01P0a2989B:0AY0912889bD0A1@B1A0A0AB033O91182440A9P8@I80n@1"+
		   "I03@1J828212A`A8:12B1@19A9@9@8^B:0@H00<82AB030bB840821Q:8310A3021"+
		   "02::A1::20A1;8"; /* OK, try to type _that_! */
		   /* [526 chars, 3156 bits] */
		/* The theory is that each bit position in each int of SPtrans is
		   set in exactly 32 entries. We keep track of set bits. */
		for (int i = 0; i < 8*64; i++)
			SPtrans[i] = 0; /* bits that are not set must be clear. */
		
		int stringpos = 0;
		for (int i = 0; i < 32; i++) /* each bit position */
		{
			int index = -1; /* pretend the -1th bit was set */
			int bit = 1 << i;
			for (int j = 0; j < 32; j++) /* each set bit */
			{
				/* Each character consists of two three-bit values: */
				int c = spt.charAt(stringpos >> 1) - '0' >> (stringpos & 1) * 3 & 7;
				stringpos++;
				if (c <= 4)
				{
					/* values 0-4 indicate a set bit 1-5 positions from the
					   previous set bit */
					SPtrans[index += c+1] |= bit;
					continue;
				}
				/* other values take at least parameter, the next value in the
				   sequence. */
				int param = spt.charAt(stringpos >> 1) - '0' >> (stringpos & 1) * 3 & 7;
				stringpos++;
				if (c == 5)
					/* 5 indicates a bit set param+6 positions from the
					   previous set bit */
					SPtrans[index += param+6] |= bit;
				else if (c == 6)
					/* 6 indicates a bit set (param*64)+1 positions from the
					   previous set bit */
					SPtrans[index += (param<<6)+1] |= bit;
				else
				{
					/* 7 indicates that we should skip (param*64) positions, then process
					   the next value which will be in the range 0-4. */
					index += param<<6;
					j--;
				}
			}
		}
	}

	//
	//  Use internal variables, makes the math a lot easier for
	//  some of the new compilers.  Might also be more efficient.
	//
	private int l, r;  // a bit disgusting, but it should be easier

	//
	// DES encrypt without IP and FP.
	// Maybe want's to be public for timing tests, but nobody's
	// that fussed in Java...
	//
	private void encrypt_base()
	{
		int u;
		int t; /* look! we fit all four variables (plus the class itself)
			into short byte-codes! */
		
		u = r << 1 | r >>> 31;
		r = l << 1 | l >>> 31;
		l = u;
		
// TEST */ System.out.println("l " + l + "\tr " + r + "       0:" + key[0]);
		for (int i = 0; i < INTERNAL_KEY_LENGTH; i += 4)
		{
			u = r^key[i  ];
			t = r^key[i+1];
			t = t >>> 4 | t << 28;
// TEST */ System.out.println("      " + i + "        u " + u + "\tt " + t);
			l^=	SPtrans[0x040 | t     &0x3f]|
				SPtrans[0x0C0 | t>> 8 &0x3f]|
				SPtrans[0x140 | t>>16 &0x3f]|
				SPtrans[0x1C0 | t>>24 &0x3f]|
				SPtrans[        u     &0x3f]|
				SPtrans[0x080 | u>> 8 &0x3f]|
				SPtrans[0x100 | u>>16 &0x3f]|
				SPtrans[0x180 | u>>24 &0x3f];
			u = l^key[i+2];
			t = l^key[i+3];
			t = t >>> 4 | t << 28;
// TEST */ System.out.println("      " + i + "        u " + u + "\tt " + t);
			r^=	SPtrans[0x040 | t     &0x3f]|
				SPtrans[0x0C0 | t>> 8 &0x3f]|
				SPtrans[0x140 | t>>16 &0x3f]|
				SPtrans[0x1C0 | t>>24 &0x3f]|
				SPtrans[        u     &0x3f]|
				SPtrans[0x080 | u>> 8 &0x3f]|
				SPtrans[0x100 | u>>16 &0x3f]|
				SPtrans[0x180 | u>>24 &0x3f];
// TEST */ System.out.println("      " + i + "        l " + l + "\tr " + r);
		}
// TEST */ System.out.println("l " + l + "\tr " + r);
		
		l = l >>> 1 | l << 31;
		r = r >>> 1 | r << 31;
		t = u = 0;
		return ; // (long)l << 32 | r & 0xFFFFFFFFL;
	}
	
	/* DES decrypt without IP and FP. */
	private void decrypt_base()
	{
		int u;
		int t; /* look! we fit all four variables (plus the class itself)
			into short byte-codes! */
		
		u = r << 1 | r >>> 31;
		r = l << 1 | l >>> 31;
		l = u;
		
		for (int i = INTERNAL_KEY_LENGTH-2; i > 0; i -= 4)
		{
			u=r^key[i  ];
			t=r^key[i+1];
			t = t >>> 4 | t << 28;
			l^=	SPtrans[0x040 | t     &0x3f]|
				SPtrans[0x0C0 | t>> 8 &0x3f]|
				SPtrans[0x140 | t>>16 &0x3f]|
				SPtrans[0x1C0 | t>>24 &0x3f]|
				SPtrans[        u     &0x3f]|
				SPtrans[0x080 | u>> 8 &0x3f]|
				SPtrans[0x100 | u>>16 &0x3f]|
				SPtrans[0x180 | u>>24 &0x3f];
			u=l^key[i-2];
			t=l^key[i-1];
			t = t >>> 4 | t << 28;
			r^=	SPtrans[0x040 | t     &0x3f]|
				SPtrans[0x0C0 | t>> 8 &0x3f]|
				SPtrans[0x140 | t>>16 &0x3f]|
				SPtrans[0x1C0 | t>>24 &0x3f]|
				SPtrans[        u     &0x3f]|
				SPtrans[0x080 | u>> 8 &0x3f]|
				SPtrans[0x100 | u>>16 &0x3f]|
				SPtrans[0x180 | u>>24 &0x3f];
		}
		
		l = l >>> 1 | l << 31;
		r = r >>> 1 | r << 31;
		t = u = 0;
		return ; // (long)l << 32 | r & 0xFFFFFFFFL;
	}
	
	/* perform IP */
	private void initialPermutation()
	{
		int t;
// TEST */  System.out.println("l " + l + "\tr " + r);
		t = (r>>>4 ^ l) & 0x0f0f0f0f;
		l ^= t;
		r ^= t << 4;
		t = (l>>>16 ^ r) & 0x0000ffff;
		r ^= t;
		l ^= t << 16;
		t = (r>>>2 ^ l) & 0x33333333;
		l ^= t;
		r ^= t << 2;
		t = (l>>>8 ^ r) & 0x00ff00ff;
		r ^= t;
		l ^= t << 8;
		t = (r>>>1 ^ l) & 0x55555555;
		l ^= t;
		r ^= t << 1;		
// TEST */  System.out.println("l " + l + "\tr " + r);
		return ; // (long)l << 32 | r & 0xFFFFFFFFL;
	}
	
	/* perform FP */
	private void finalPermutation()
	{
		int t;
		t = (r>>>1 ^ l) & 0x55555555;
		l ^= t;
		r ^= t << 1;
		t = (l>>>8 ^ r) & 0x00ff00ff;
		r ^= t;
		l ^= t << 8;
		t = (r>>>2 ^ l) & 0x33333333;
		l ^= t;
		r ^= t << 2;
		t = (l>>>16 ^ r) & 0x0000ffff;
		r ^= t;
		l ^= t << 16;
		t = (r>>>4 ^ l) & 0x0f0f0f0f;
		l ^= t;
		r ^= t << 4;
		return ; // (long)l << 32 | r & 0xFFFFFFFFL;
	}
	
	//
	// Encrypt a block.
	// @param inbuf the block to encrypt (of length BLOCK_LENGTH)
	// @param outbuf the encrypted block (of length BLOCK_LENGTH)
	// @param enc encrypt or decrypt
	//
	private void java_encrypt(byte inbuf[], int in_offset, byte outbuf[], int out_offset, boolean enc)
	{
		// long lr; // longs mean trouble in Java
		// int[] lr = new int[2]; // this is more trouble as well
		
		l = r = 0;
		for (int i = 3; i >= 0; i--)
		{
			l <<= 8;
			l |= inbuf[in_offset+i] & 0xFF;
		}
		for (int i = 7; i >= 4; i--)
		{
			r <<= 8;
			r |= inbuf[in_offset+i] & 0xFF;
		}
// TEST */  System.out.println("LR " + l + "   " + r);

		// this method doesn't work with jolt guava compiler
		// l =
		//	(inbuf[in_offset+0] & 0xFF) |
		//	(inbuf[in_offset+1] << 8 & 0xFF00) |
		//	(inbuf[in_offset+2] << 16 & 0xFF0000) |
		//	(inbuf[in_offset+3] << 24 & 0xFF000000);
		// r =
		//	(inbuf[in_offset+4] & 0xFF) |
		//	(inbuf[in_offset+5] << 8 & 0xFF00) |
		//	(inbuf[in_offset+6] << 16 & 0xFF0000) |
		//	(inbuf[in_offset+7] << 24 & 0xFF000000);
		// System.out.println("LR " + l + "   " + r);

		initialPermutation();

		/* do it! */
// TEST */  System.out.println("IP " + l + "   " + r);
		if (enc)
			encrypt_base(); // (int)(lr >> 32),(int)lr);
		else
			decrypt_base(); // (int)(lr >> 32),(int)lr);
			
// TEST */  System.out.println("en " + l + "   " + r);
		/* FP */
		finalPermutation(); // (int)(lr >> 32),(int)lr);
// TEST */  System.out.println("FP " + l + "   " + r);
		
		outbuf[out_offset+0] = (byte)(l    );
		outbuf[out_offset+1] = (byte)(l>>8 );
		outbuf[out_offset+2] = (byte)(l>>16);
		outbuf[out_offset+3] = (byte)(l>>24);
		outbuf[out_offset+4] = (byte)(r    );
		outbuf[out_offset+5] = (byte)(r>> 8);
		outbuf[out_offset+6] = (byte)(r>>16);
		outbuf[out_offset+7] = (byte)(r>>24);

// TEST */  System.out.println("================================");
		l = r = 0;
	}



// // // // // // // // //  T E S T  // // // // // // // // // // // // 

	//
	// Don't expand this code please without thinking about it,
	// much better to write a separate program.
	//
	/**
	 * Entry point for very basic <code>self_test</code>.
	 */
	public static final void
	main(String argv[]) 
	{
		try { self_test(); }
		catch(Throwable t) { t.printStackTrace(); }
	}


//
// Not sure if these are official, probably not.
// Use ints as Java grumbles about negative hex values.
//
static final private byte[][][] tests =
{
  {
    { 1, 1, 1, 1, 1, 1, 1, 1, },
    { -107, -8, -91, -27, -35, 49, -39, 0, },
    {  -128, 0, 0, 0, 0, 0, 0, 0, },
  },
  {
    { 1, 1, 1, 1, 1, 1, 1, 2, },
    { 0, 0, 0, 0, 0, 0, 0, 0, },
    { -122, -98, -3, 127, -97, 38, 90, 9, },
  },
  {
    { 56, 73, 103, 76, 38, 2, 49, -98, },
    { 81, 69, 75, 88, 45, -33, 68, 10, },
    { 113, 120, -121, 110, 1, -15, -101, 42, },
  },
};

/**
 * Do some basic tests.
 * Three of the certification data are included only, no output,
 * success or exception.
 * If you want more, write a test program!
 * @see cryptix.examples.DES
 */
public static final void
self_test()
	throws CryptoError
{
	for (int i = 0; i < tests.length; i++)
	{
		byte[][] test = tests[i];
		byte[] key    = test[0];
		byte[] plain  = test[1];
		byte[] cipher = test[2];

		DES cryptor = new DES(key);
		byte[] tmp = new byte[cryptor.blockLength()];

		cryptor.encrypt(plain, tmp);
		if (notEquals(cipher, tmp))
			throw new CryptoError("encrypt #"+ i +" failed");
		cryptor.decrypt(cipher, tmp);
		if (notEquals(plain, tmp))
			throw new CryptoError("decrypt #"+ i +" failed");
	}
}
private static boolean
notEquals(byte[] a, byte[] b)
{
	for (int i = 0; i < a.length; i++)
	{
		if (a[i] != b[i])
			return true;
		// if (a[i] == 88) // force a trip up
		// 	return true;
	}
	return false;
}


} /* class DES */

/* The DES cipher (in ECB mode).
 *
 * Parts of this code Copyright (C) 1996 Geoffrey Keating. All rights reserved.
 *
 * Its use is FREE FOR COMMERCIAL AND NON-COMMERCIAL USE
 * as long as the following conditions are adhered to.
 * 
 * Copyright remains Geoffrey Keating's, and as such any Copyright notices in
 * the code are not to be removed.  If this code is used in a product,
 * Geoffrey Keating should be given attribution as the author of the parts used.
 * This can be in the form of a textual message at program startup or
 * in documentation (online or textual) provided with the package.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgement:
 *    This product includes software developed by Eric Young (eay@mincom.oz.au)
 * 
 * THIS SOFTWARE IS PROVIDED BY GEOFFREY KEATING ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * 
 */
/* Parts of this code (in particular, the string representing SPtrans below)
 * are Copyright (C) 1995 Eric Young (eay@mincom.oz.au). All rights reserved.
 * Its use is 
 * FREE FOR COMMERCIAL AND NON-COMMERCIAL USE
 * as long as the following conditions are adhered to.
 * 
 * Copyright remains Eric Young's, and as such any Copyright notices in
 * the code are not to be removed.  If this code is used in a product,
 * Eric Young should be given attribution as the author of the parts used.
 * This can be in the form of a textual message at program startup or
 * in documentation (online or textual) provided with the package.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgement:
 *    This product includes software developed by Eric Young (eay@mincom.oz.au)
 * 
 * THIS SOFTWARE IS PROVIDED BY ERIC YOUNG ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */
