/*
 * Copyright (C) 1995, 1996 Systemics Ltd (http://www.systemics.com/)
 * All rights reserved.
 *
 * This library and applications are FREE FOR COMMERCIAL AND NON-COMMERCIAL USE
 * as long as the conditions within the COPYRIGHT file are adhered to.
 *
 */

package systemics.util;

import java.util.Hashtable;
import java.util.Enumeration;
import java.io.*;


/**
 * This class will read and write windows style(ish) ini files
 * It has a multi line extension if you enter <pre>item = << END<pre><p>
 * then all the lines up until the line <pre>END<pre> will be readin as one line.
 *
 */

public class IniFile
{
	protected Hashtable data = new Hashtable();

	protected String iniFilename;

	public IniFile( String filename )
	throws FileNotFoundException, IOException
	{
		iniFilename = filename;
		FileInputStream inifile = new FileInputStream( iniFilename );
		load( new DataInputStream( inifile ) );
        		inifile.close();
		// Shoud filename be stored, and used in the destructor or save function to rewrite the file?
	}

	public synchronized void 
	load( InputStream raw_in )
	throws IOException
	{
		DataInputStream in = ( raw_in instanceof DataInputStream ) ? (DataInputStream)raw_in : new DataInputStream( raw_in );
		Hashtable h = null, data = this.data = new Hashtable();
		String s, end = null, item = null;
		boolean multiline = false;
		StringBuffer sb = null;
		read:
		while ( ( s = in.readLine() ) != null )
		{
			if ( !multiline )
			{
				int i;
				if ( ( s = s.trim() ).length() == 0 )
					continue;
				switch ( s.charAt( 0 ) )
				{
				case '[':
					if ( ( i = s.indexOf( "]" ) ) == - 1 )
						throw new IniFileFormatException( "Invalid field definition " + s );
					data.put( s.substring( 1, i ), h = new Hashtable() );
					continue read;
				case ';': // comments
				case '#':
					continue read;
				default:
					if ( h == null )
						throw new IniFileFormatException( "Missing field definition" );
					if ( ( i = s.indexOf( "=" ) ) == - 1 )
						throw new IniFileFormatException( "Invalid field definition " + s );
					item  = s.substring( 0, i ).trim();
					s = s.substring( i + 1 ).trim();
					if ( !s.startsWith( "<<" ) )
					{
						h.put( item, s );
						continue read;
					}
					if ( ( end = s.substring( 2 ).trim() ).length() == 0 ) 
						throw new IniFileFormatException( "Missing end definition" );
					multiline = true;
					sb = null;
					continue read;
				}
			}
			// multi line
			if ( end.equals( s.trim() ) )
			{
				if ( h == null )
					throw new IniFileFormatException( "Missing field definition" );
				h.put( item, sb.toString() );
				multiline = false;
			}
			if ( sb == null )
				sb = new StringBuffer();
			else
				sb.append( '\n' );
			sb.append( s );
		}
	}
	
	public synchronized void 
	save( OutputStream raw_out, String header )
	{
		PrintStream out = ( raw_out instanceof PrintStream ) ? (PrintStream)raw_out : new PrintStream( raw_out );
		Hashtable data = this.data;
		for (Enumeration e = data.keys(); e.hasMoreElements(); )
		{
			String field;
			Hashtable h = (Hashtable)data.get( field = (String)e.nextElement() );
			out.println( "[" + field + "]" );
			for ( Enumeration items = h.keys(); items.hasMoreElements(); )
			{
				String item;
				String value = (String)h.get( item = (String)items.nextElement() );
				if ( value.indexOf( "\n" ) == -1 ) // single line
				{
					out.println( item + "=" + value );
					continue;
				}
				out.println( item + "=<<END\n" + value + "\nEND" );
			}
		}
	}
	
	public synchronized String
	get( String field, String item )
	{
		return get( field, item, null );
	}

	public synchronized String
	get( String field, String item, String def )
	{
		Hashtable h;
		if ( ( h = (Hashtable)data.get( field ) ) != null )
		{
			String s;
			if ( ( s = (String)h.get( item ) ) != null )
				return s;
		}
		return def;

	}

	public final synchronized String
	set( String field, String item, String value )
	{
		Hashtable h;
		if ( ( h = (Hashtable)data.get( field ) ) == null )
			data.put( field, h = new Hashtable() );
		return (String)h.put( item, value );
	}
	
	public final synchronized boolean
	fieldExists( String field )
	{
		return data.get( field ) != null;
	}
	
	public final String
	name()
	{
		return iniFilename;
	}
}

