package systemics.util;

import java.io.DataInputStream;
import java.io.FileInputStream;
import java.io.IOException;
import java.io.FileNotFoundException;

/**
 * Assumes the preferences file can change at any time,
 * so re-loads it every time a program requests data
 * from it.
 */
public class VolatileIniFile extends IniFile
{
	public VolatileIniFile( String filename )
	throws FileNotFoundException, IOException
	{
		super( filename );
	}

	public String get( String section, String option )
	{
		reload();
		return get( section, option );
	}

	public String get( String section, String option, String defaultValue )
	{
		reload();
		return get( section, option, defaultValue );
        }

	protected void reload()
	{
		try
		{
			load( new DataInputStream( new FileInputStream( name() ) ) );
		}

		// Isn't a huge problem if file can't be reloaded - just notify user.
		catch( IOException ioe )
		{
			System.err.println( "Couldn't reload preferences file " + name() );
			System.err.println( ioe.getMessage() );
		}
	}
}
